<?php
class QUESANS_BOL_TopicDao extends OW_BaseDao
{
    const GROUP_ID = 'groupId';
    const STATUS = 'status';
    /**
     * Class constructor
     *
     */
    protected function __construct()
    {
        parent::__construct();
    }
    /**
     * Class instance
     *
     * @var QUESANS_BOL_TopicDao
     */
    private static $classInstance;

    /**
     * Returns class instance
     *
     * @return QUESANS_BOL_TopicDao
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    /**
     * @see OW_BaseDao::getDtoClassName()
     *
     */
    public function getDtoClassName()
    {
        return 'QUESANS_BOL_Topic';
    }

    /**
     * @see OW_BaseDao::getTableName()
     *
     */
    public function getTableName()
    {
        return OW_DB_PREFIX . 'quesans_topic';
    }

    /**
     * Returns quesans group's topic count
     * 
     * @param int 
     * @return int $groupId
     */
    public function findGroupTopicCount( $groupId )
    {
        if ( empty($groupId) )
        {
            return 0;
        }

        $example = new OW_Example();

        $example->andFieldEqual(self::GROUP_ID, (int)$groupId);
        $example->andFieldEqual(self::STATUS, QUESANS_BOL_QuesansService::STATUS_APPROVED);

        return $this->countByExample($example);
    }

    /**
     * Returns QUESANS group's post count
     * 
     * @param int $groupId
     * @return int
     */
    public function findGroupPostCount( $groupId )
    {
        if ( empty($groupId) )
        {
            return 0;
        }

        $query = 'SELECT COUNT(`p`.`id`)
            FROM `' . $this->getTableName() . '` AS `t`
		        INNER JOIN `' . QUESANS_BOL_PostDao::getInstance()->getTableName() . '` AS `p`
		            ON ( `t`.`id` = `p`.`topicId` )
		    WHERE `t`.`' . self::GROUP_ID . '` = :groupId AND `t`.`' . self::STATUS . '` = :status';

        return (int)$this->dbo->queryForColumn($query, array('groupId' => $groupId, 'status' => QUESANS_BOL_QuesansService::STATUS_APPROVED));
    }

    /**
     * Returns QUESANS group's topic list
     * 
     * @param int $groupId
     * @param int $first
     * @param int $count
     * @return array 
     */
    public function findGroupTopicList( $groupId, $first, $count )
    {
        $query = 'SELECT `t`.*
		    FROM `' . $this->getTableName() . '` AS `t`
		        INNER JOIN `' . QUESANS_BOL_PostDao::getInstance()->getTableName() . '` AS `p`
		            ON (`t`.`lastPostId` = `p`.`id`)
		    WHERE `t`.`groupId` = ? AND `t`.`status` = ?
		    GROUP BY `p`.`topicId`
		    ORDER BY `t`.`sticky` DESC, `p`.`createStamp` DESC
		    LIMIT ?, ?';

        $list = $this->dbo->queryForList($query, array($groupId, QUESANS_BOL_QuesansService::STATUS_APPROVED, (int)$first, (int)$count));

        if ( $list )
        {
            $topicIdList = array();
            foreach ( $list as $topic )
            {
                $topicIdList[] = $topic['id'];
            }

            $counters = $this->getPostCountForTopicIdList($topicIdList);
            foreach ( $list as &$topic )
            {
                $topic['postCount'] = $counters[$topic['id']];
            }
        }

        return $list;
    }

    public function findLastTopicList( $limit, $excludeGroupIdList = null )
    {
        $postDao = QUESANS_BOL_PostDao::getInstance();
        $groupDao = QUESANS_BOL_GroupDao::getInstance();
        $sectionDao = QUESANS_BOL_SectionDao::getInstance();

        $excludeCond = $excludeGroupIdList ? ' AND `g`.`id` NOT IN ('.implode(',', $excludeGroupIdList).') = 1' : '';

        $query = 'SELECT `t`.*
            FROM `' . $this->getTableName() . '` AS `t`
                INNER JOIN `' . $groupDao->getTableName() . '` AS `g` ON (`t`.`groupId` = `g`.`id`)
                INNER JOIN `' . $sectionDao->getTableName() . '` AS `s` ON (`s`.`id` = `g`.`sectionId`)
                INNER JOIN `' . $postDao->getTableName() . '` AS `p` ON (`t`.`lastPostId` = `p`.`id`)
            WHERE `s`.`isHidden` = 0 AND `t`.`status` = :status ' . $excludeCond . '
            ORDER BY `p`.`createStamp` DESC
            LIMIT :limit';

        $list = $this->dbo->queryForList($query, array('status' => QUESANS_BOL_QuesansService::STATUS_APPROVED, 'limit' => (int)$limit));

        if ( $list )
        {
            $topicIdList = array();
            foreach ( $list as $topic )
            {
                $topicIdList[] = $topic['id'];
            }

            $counters = $this->getPostCountForTopicIdList($topicIdList);
            foreach ( $list as &$topic )
            {
                $topic['postCount'] = $counters[$topic['id']];
            }
        }

        return $list;
    }

    public function getPostCountForTopicIdList( $topicIdList )
    {
        $postDao = QUESANS_BOL_PostDao::getInstance();

        $query = "SELECT `p`.`topicId`, COUNT(`p`.`id`) AS `postCount`
            FROM `".$postDao->getTableName()."` AS `p`
            WHERE `p`.`topicId` IN (".$this->dbo->mergeInClause($topicIdList).")
            GROUP BY `p`.`topicId`";

        $countList = $this->dbo->queryForList($query);

        $counters = array();
        foreach ( $countList as $count )
        {
            $counters[$count['topicId']] = $count['postCount'];
        }

        return $counters;
    }

    public function findUserTopicList( $userId )
    {
        $query = "
            SELECT * FROM `" . $this->getTableName() . "` WHERE `userId` = ?
        ";

        return $this->dbo->queryForList($query, array($userId));
    }

    /**
     * Returns quesans topic info
     * 
     * @param int $topicId
     * @return array 
     */
    public function findTopicInfo( $topicId )
    {
        $query = "
		SELECT `t`.*, `g`.`id` AS `groupId`, `g`.`name` AS `groupName`, `s`.`name` AS `sectionName`, `s`.`id` AS `sectionId` 
		FROM `" . $this->getTableName() . "` AS `t`
		LEFT JOIN `" . QUESANS_BOL_GroupDao::getInstance()->getTableName() . "` AS `g` 
		ON (`t`.`groupId` = `g`.`id`)
		LEFT JOIN `" . QUESANS_BOL_SectionDao::getInstance()->getTableName() . "` AS `s`
		ON (`g`.`sectionId` = `s`.`id`)
		WHERE `t`.`id` = ?
		";

        return $this->dbo->queryForRow($query, array($topicId));
    }

    /**
     * Returns topic id list
     * 
     * @param array $groupIds
     * @return array 
     */
    public function findIdListByGroupIds( $groupIds )
    {
        $example = new OW_Example();
        $example->andFieldInArray('groupId', $groupIds);

        $query = "
    	SELECT `id` FROM `" . $this->getTableName() . "`
    	" . $example;

        return $this->dbo->queryForColumnList($query);
    }

    public function getTopicIdListForDelete( $limit )
    {
        $example = new OW_Example();
        $example->setOrder('`id` ASC');
        $example->setLimitClause(0, $limit);

        return $this->findIdListByExample($example);
    }
    
    public function findTemporaryTopicList( $limit )
    {
        $postDao = QUESANS_BOL_PostDao::getInstance();
        
        $query = "SELECT `t`.* FROM `".$this->getTableName()."` AS `t`
            LEFT JOIN `".$postDao->getTableName()."` AS `p` ON (`t`.`lastPostId`=`p`.`id`)
            WHERE `t`.`temp` = 1 AND `p`.`createStamp` < :ts";
        
        return $this->dbo->queryForList($query, array('ts' => time() - 3600 * 24 * 5));
    }
}