<?php
class QUESANS_CLASS_EventHandler
{
    /**
     * @var QUESANS_CLASS_EventHandler
     */
    private static $classInstance;

    /**
     * @return QUESANS_CLASS_EventHandler
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    public function __construct() { }

    public function addNewContentItem( BASE_CLASS_EventCollector $event )
    {
        $resultArray = array(
            BASE_CMP_AddNewContent::DATA_KEY_ICON_CLASS => 'ow_ic_files',
            BASE_CMP_AddNewContent::DATA_KEY_URL => OW::getRouter()->urlForRoute('add-topic-default'),
            BASE_CMP_AddNewContent::DATA_KEY_LABEL => OW::getLanguage()->text('quesans', 'discussion')
        );

        if ( !OW::getUser()->isAuthorized('quesans', 'edit') )
        {
            $status = BOL_AuthorizationService::getInstance()->getActionStatus('quesans', 'edit');

            if ( $status['status'] != BOL_AuthorizationService::STATUS_PROMOTED )
            {
                return;
            }

            $id = uniqid('add-new-quesans-');
            $resultArray[BASE_CMP_AddNewContent::DATA_KEY_ID] = $id;

            $script = '$("#'.$id.'").click(function(){
                OW.authorizationLimitedFloatbox('.json_encode($status['msg']).');
                return false;
            });';
            OW::getDocument()->addOnloadScript($script);
        }

        $event->add($resultArray);
    }

    public function deleteUserContent( OW_Event $event )
    {
        $params = $event->getParams();

        if ( !isset($params['deleteContent']) || !(bool) $params['deleteContent'] )
        {
            return;
        }

        $userId = (int) $params['userId'];

        if ( $userId > 0 )
        {
            $quesansService = QUESANS_BOL_QuesansService::getInstance();

            $quesansService->deleteUserTopics($userId);
            $quesansService->deleteUserPosts($userId);
        }
    }

    public function createSection( OW_Event $event )
    {
        $params = $event->getParams();

        if ( !isset($params['name']) || !isset($params['entity']) || !isset($params['isHidden']) )
        {
            return;
        }

        $quesans_service = QUESANS_BOL_QuesansService::getInstance();

        $sectionDto = $quesans_service->findSectionByEntity($params['entity']);

        if ( !isset($sectionDto) )
        {
            $sectionDto = new QUESANS_BOL_Section();
            $sectionDto->name = $params['name'];
            $sectionDto->entity = $params['entity'];
            $sectionDto->isHidden = $params['isHidden'];
            $sectionDto->order = $quesans_service->getNewSectionOrder();

            $quesans_service->saveOrUpdateSection($sectionDto);
        }

    }

    public function deleteSection( OW_Event $event )
    {
        $params = $event->getParams();

        if ( !isset($params['name']) && !isset($params['entity']) )
        {
            return;
        }

        $quesans_service = QUESANS_BOL_QuesansService::getInstance();

        if ( isset($params['name']) )
        {
            $section = $quesans_service->getSection($params['name']);
        }

        if ( isset($params['entity']) )
        {
            $section = $quesans_service->findSectionByEntity($params['entity']);
        }

        if ( !empty($section) )
        {
            $quesans_service->deleteSection($section->getId());
        }
    }

    public function addWidget( OW_Event $event )
    {
        $params = $event->getParams();

        if ( !isset($params['place']) || !isset($params['section']) )
        {
            return;
        }

        try
        {
            $widgetService = BOL_ComponentAdminService::getInstance();
            $widget = $widgetService->addWidget('QUESANS_CMP_LatestTopicsWidget', false);
            $placeWidget = $widgetService->addWidgetToPlace($widget, $params['place']);
            $widgetService->addWidgetToPosition($placeWidget, $params['section'], 0);
        }
        catch ( Exception $e ) { }
    }

    public function installWidget( OW_Event $e )
    {
        $params = $e->getParams();

        $widgetService = BOL_ComponentAdminService::getInstance();

        try
        {
            $widget = $widgetService->addWidget('QUESANS_CMP_LatestTopicsWidget', false);
            $widgetPlace = $widgetService->addWidgetToPlace($widget, $params['place']);
            $widgetService->addWidgetToPosition($widgetPlace, $params['section'], $params['order']);
            $e->setData($widgetPlace->uniqName);
        }
        catch ( Exception $exception )
        {
            $e->setData(false);
        }
    }

    public function deleteWidget( OW_Event $event )
    {
        BOL_ComponentAdminService::getInstance()->deleteWidget('QUESANS_CMP_LatestTopicsWidget');
    }

    public function createGroup( OW_Event $event )
    {
        $params = $event->getParams();

        if ( !$params['entity'] || !isset($params['name']) || !isset($params['description']) || !isset($params['entityId']) )
        {
            return;
        }

        $quesansService = QUESANS_BOL_QuesansService::getInstance();

        $quesansGroup = $quesansService->findGroupByEntityId($params['entity'], $params['entityId']);

        if ( !isset($quesansGroup) )
        {
            $section = $quesansService->findSectionByEntity($params['entity']);

            $quesansGroup = new QUESANS_BOL_Group();
            $quesansGroup->sectionId = $section->getId();
            $quesansGroup->order = $quesansService->getNewGroupOrder($section->getId());

            $quesansGroup->name = $params['name'];
            $quesansGroup->description = $params['description'];
            $quesansGroup->entityId = $params['entityId'];

            $quesansService->saveOrUpdateGroup($quesansGroup);
        }
    }

    public function deleteGroup( OW_Event $event )
    {
        $params = $event->getParams();

        if ( !isset($params['entityId']) || !isset($params['entity']) )
        {
            return;
        }

        $quesansService = QUESANS_BOL_QuesansService::getInstance();
        $group = $quesansService->findGroupByEntityId($params['entity'], $params['entityId']);

        if ( !empty($group) )
        {
            $quesansService->deleteGroup($group->getId());
        }
    }

    public function editGroup( OW_Event $event )
    {
        $params = $event->getParams();

        if ( !isset($params['entityId']) || !isset($params['entity']) )
        {
            return;
        }

        $quesansService = QUESANS_BOL_QuesansService::getInstance();
        $group = $quesansService->findGroupByEntityId($params['entity'], $params['entityId']);

        if ( !empty($group) )
        {
            if (!empty($params['name']))
            {
                $group->name = $params['name'];
            }

            if (!empty($params['description']))
            {
                $group->description = $params['description'];
            }

            $quesansService->saveOrUpdateGroup($group);
        }
    }

    public function onNotifyActions( BASE_CLASS_EventCollector $e )
    {
        $e->add(array(
            'section' => 'quesans',
            'action' => 'quesans-add_post',
            'sectionIcon' => 'ow_ic_quesans',
            'sectionLabel' => OW::getLanguage()->text('quesans', 'email_notifications_section_label'),
            'description' => OW::getLanguage()->text('quesans', 'email_notifications_setting_post'),
            'selected' => true
        ));
    }

    public function addPost( OW_Event $e )
    {
        $params = $e->getParams();

        $postId = $params['postId'];

        $quesansService = QUESANS_BOL_QuesansService::getInstance();
        $post = $quesansService->findPostById($postId);

        if ( !$post )
        {
            return;
        }

        $userIdList = QUESANS_BOL_SubscriptionService::getInstance()->findTopicSubscribers($post->topicId);
        if ( empty($userIdList) )
        {
            return;
        }

        $params = array(
            'pluginKey' => 'quesans',
            'entityType' => 'quesans_topic_reply',
            'entityId' => $postId,
            'action' => 'quesans-add_post',
            'time' => time()
        );

        $authorId = $post->userId;
        $avatar = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($authorId));
        $postUrl = $quesansService->getPostUrl($post->topicId, $postId);
        $topicUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $post->topicId));
        $topic = $quesansService->findTopicById($post->topicId);

        $data = array(
            'avatar' => $avatar[$authorId],
            'string' => array(
                'key' => 'quesans+email_notification_post',
                'vars' => array(
                    'userName' => $avatar[$authorId]['title'],
                    'userUrl' => $avatar[$authorId]['url'],
                    'postUrl' => $postUrl,
                    'topicUrl' => $topicUrl,
                    'title' => strip_tags($topic->title)
                )
            ),
            'content' => strip_tags($post->text),
            'url' => $postUrl
        );

        foreach ( $userIdList as $userId )
        {
            if ( $userId == $post->userId )
            {
                continue;
            }

            $params['userId'] = $userId;

            $event = new OW_Event('notifications.add', $params, $data);
            OW::getEventManager()->trigger($event);
        }
    }

    public function adsEnabled( BASE_CLASS_EventCollector $event )
    {
        $event->add('quesans');
    }

    public function addAuthLabels( BASE_CLASS_EventCollector $event )
    {
        $language = OW::getLanguage();
        $event->add(
            array(
                'quesans' => array(
                    'label' => $language->text('quesans', 'auth_group_label'),
                    'actions' => array(
                        'edit' => $language->text('quesans', 'auth_action_label_edit'),
                        'view' => $language->text('quesans', 'auth_action_label_view'),
                        'subscribe' => $language->text('quesans', 'auth_action_label_subscribe'),
                        'move_topic_to_hidden' => $language->text('quesans', 'auth_action_label_move_topic_to_hidden')
                    )
                )
            )
        );
    }

    public function feedOnEntityAdd( OW_Event $e )
    {
        $params = $e->getParams();
        $data = $e->getData();

        if ( $params['entityType'] != 'quesans-topic' )
        {
            return;
        }

        $topicId = (int) $params['entityId'];
        $service = QUESANS_BOL_QuesansService::getInstance();
        $topicDto = $service->findTopicById($topicId);
        $postDto = $service->findTopicFirstPost($topicId);
        $groupDto = $service->findGroupById($topicDto->groupId);
        $sectionDto = $service->findSectionById($groupDto->sectionId);
        $isHidden = (bool) $sectionDto->isHidden;

        if ( $postDto === null )
        {
            return;
        }

        if ( $groupDto->isPrivate )
        {
            return;
        }

        $topicUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicDto->id));
        $content = UTIL_String::truncate(strip_tags($postDto->text), 150, '...');
        $title = UTIL_String::truncate(strip_tags($topicDto->title), 100, '...');

        $data = array(
            'features' => array('likes'),
            'ownerId' => $topicDto->userId,
            'time' => (int) $postDto->createStamp,
            'content' => array(
                'format' => 'content',
                'vars' => array(
                    'title' => $title,
                    'description' => $content,
                    'url' => array(
                        "routeName" => 'topic-default',
                        "vars" => array('topicId' => $topicDto->id)
                    ),
                    'iconClass' => 'ow_ic_quesans'
                )
            ),
            'string' => array('key' => 'quesans+feed_activity_topic_string'),
            'toolbar' => array(array(
                'href' => $topicUrl,
                'label' => OW::getLanguage()->text('quesans', 'feed_toolbar_discuss')
            ))
        );

        if ( $isHidden )
        {
            $data['params']['feedType'] = $sectionDto->entity;
            $data['params']['feedId'] = $groupDto->entityId;
            $data['params']['visibility'] = 2 + 4 + 8; // Visible for follows(2), autor (4) and current feed (8)
            $data['params']['postOnUserFeed'] = false;
            $data['contextFeedType'] = $data['params']['feedType'];
            $data['contextFeedId'] = $data['params']['feedId'];
        }

        $e->setData($data);
    }

    public function feedOnPostAdd( OW_Event $e )
    {
        $params = $e->getParams();

        $event = new OW_Event('feed.activity', array(
            'pluginKey' => 'quesans',
            'entityType' => 'quesans-topic',
            'entityId' => $params['topicId'],
            'userId' => $params['userId'],
            'activityType' => 'quesans-post',
            'activityId' => $params['postId'],
            'subscribe' => true
        ), array(
            'postId' => $params['postId'],
            'string' => array('key' => 'quesans+feed_activity_topic_reply_string')
        ));
        OW::getEventManager()->trigger($event);
    }

    public function feedOnItemRender( OW_Event $event )
    {
        $params = $event->getParams();
        $data = $event->getData();
        $language = OW::getLanguage();

        if ( $params['action']['entityType'] != 'quesans-topic' )
        {
            return;
        }

        $service = QUESANS_BOL_QuesansService::getInstance();
        $postCount = $service->findTopicPostCount($params['action']['entityId']) - 1;

        if ( !$postCount )
        {
            return;
        }

        if ( is_array($data['toolbar']) )
        {
            $data['toolbar'][] = array(
                'label' => $language->text('quesans', 'feed_toolbar_replies', array('postCount' => $postCount))
            );
        }

        $event->setData($data);

        $postIds = array();
        foreach ( $params['activity'] as $activity )
        {
            if ( $activity['activityType'] == 'quesans-post' )
            {
                $postIds[] = $activity['data']['postId'];
            }
        }

        if ( empty($postIds) )
        {
            return;
        }

        $postDto = null;
        foreach ( $postIds as $pid )
        {
            $postDto = $service->findPostById($pid);
            if ( $postDto !== null )
            {
                break;
            }
        }

        if ( $postDto === null )
        {
            return;
        }

        $postUrlEmbed = '...';
        $content = UTIL_String::truncate(strip_tags(str_replace("&nbsp;", "", $postDto->text)), 100, $postUrlEmbed);
        $usersData = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($postDto->userId), true, true, true, false);

        $avatarData = $usersData[$postDto->userId];
        $postUrl = $service->getPostUrl($postDto->topicId, $postDto->id);

        if ( is_array($data['content']) && !empty($data['content']['vars']) )
        {
            $data['content']['vars']['activity'] = array(
                'url' => $postUrl,
                'title' => $language->text('quesans', 'last_reply'),
                'avatarData' => $avatarData,
                'description' => $content
            );
        }

        $event->setData($data);
    }

    public function feedCollectConfigurableActivity( BASE_CLASS_EventCollector $event )
    {
        $language = OW::getLanguage();
        $event->add(array(
            'label' => $language->text('quesans', 'feed_content_label'),
            'activity' => 'create:quesans-topic'
        ));

        $event->add(array(
            'label' => $language->text('quesans', 'feed_content_replies_label'),
            'activity' => 'quesans-post:quesans-topic'
        ));
    }

    public function subscribeUser( OW_Event $e )
    {
        $params = $e->getParams();
        $userId = (int) $params['userId'];
        $topicId = (int) $params['topicId'];

        if ( !$userId || ! $topicId )
        {
            return false;
        }

        $service = QUESANS_BOL_SubscriptionService::getInstance();

        if ( $service->isUserSubscribed($userId, $topicId) )
        {
            return true;
        }

        $subs = new QUESANS_BOL_Subscription();
        $subs->userId = $userId;
        $subs->topicId = $topicId;

        $service->addSubscription($subs);

        return true;
    }

    public function feedTopicLike( OW_Event $event )
    {
        $params = $event->getParams();

        if ( $params['entityType'] != 'quesans-topic' )
        {
            return;
        }

        $service = QUESANS_BOL_QuesansService::getInstance();
        $topic = $service->findTopicById($params['entityId']);
        $userId = $topic->userId;

        $userName = BOL_UserService::getInstance()->getDisplayName($userId);
        $userUrl = BOL_UserService::getInstance()->getUserUrl($userId);
        $userEmbed = '<a href="' . $userUrl . '">' . $userName . '</a>';

        if ( $params['userId'] == $userId )
        {
            $string = array('key' => 'quesans+feed_activity_owner_topic_like');
        }
        else
        {
            $string = array('key' => 'quesans+feed_activity_topic_string_like', 'vars' => array('user' => $userEmbed));
        }

        OW::getEventManager()->trigger(new OW_Event('feed.activity', array(
            'activityType' => 'like',
            'activityId' => $params['userId'],
            'entityId' => $params['entityId'],
            'entityType' => $params['entityType'],
            'userId' => $params['userId'],
            'pluginKey' => 'photo'
        ), array(
            'string' => $string
        )));
    }

    public function init()
    {
        $this->genericInit();
        $em = OW::getEventManager();

        $em->bind(BASE_CMP_AddNewContent::EVENT_NAME, array($this, 'addNewContentItem'));
        $em->bind('quesans.add_widget', array($this, 'addWidget'));
        $em->bind('quesans.install_widget', array($this, 'installWidget'));
        $em->bind('quesans.delete_widget', array($this, 'deleteWidget'));
        $em->bind('feed.on_item_render', array($this, 'feedOnItemRender'));
    }

    public function sosialSharingGetQuesansInfo( OW_Event $event )
    {
        $params = $event->getParams();
        $data = $event->getData();
        $data['display'] = false;

        if ( empty($params['entityId']) )
        {
            return;
        }

        if ( $params['entityType'] == 'quesans_topic' )
        {
            $topicDto = QUESANS_BOL_QuesansService::getInstance()->findTopicById($params['entityId']);

            $quesansGroup = QUESANS_BOL_QuesansService::getInstance()->findGroupById($topicDto->groupId);
            $quesansSection = QUESANS_BOL_QuesansService::getInstance()->findSectionById($quesansGroup->sectionId);

            if ( !empty($topicDto) )
            {
                $data['display'] = !$quesansSection->isHidden && !$quesansGroup->isPrivate && BOL_AuthorizationService::getInstance()->isActionAuthorizedForGuest('quesans', 'view');
            }
            
            $event->setData($data);
        }
    }

    public function genericInit()
    {
        $em = OW::getEventManager();

        $em->bind(OW_EventManager::ON_USER_UNREGISTER, array($this, 'deleteUserContent'));
        $em->bind('quesans.create_section', array($this, 'createSection'));
        $em->bind('quesans.delete_section', array($this, 'deleteSection'));
        $em->bind('quesans.create_group', array($this, 'createGroup'));
        $em->bind('quesans.delete_group', array($this, 'deleteGroup'));
        $em->bind('quesans.edit_group', array($this, 'editGroup'));
        $em->bind('notifications.collect_actions', array($this, 'onNotifyActions'));
        $em->bind('quesans.add_post', array($this, 'addPost'));
        $em->bind('ads.enabled_plugins', array($this, 'adsEnabled'));
        $em->bind('admin.add_auth_labels', array($this, 'addAuthLabels'));
        $em->bind('feed.on_entity_add', array($this, 'feedOnEntityAdd'));
        $em->bind('feed.on_entity_update', array($this, 'feedOnEntityAdd'));
        $em->bind('quesans.add_post', array($this, 'feedOnPostAdd'));
        $em->bind('feed.collect_configurable_activity', array($this, 'feedCollectConfigurableActivity'));
        $em->bind('quesans.subscribe_user', array($this, 'subscribeUser'));
        $em->bind('feed.after_like_added', array($this, 'feedTopicLike'));

        $credits = new QUESANS_CLASS_Credits();
        $em->bind('usercredits.on_action_collect', array($credits, 'bindCreditActionsCollect'));
        $em->bind('usercredits.get_action_key', array($credits, 'getActionKey'));

        $em->bind('socialsharing.get_entity_info', array($this, 'sosialSharingGetQuesansInfo'));
    }
}