<?php
class QUESANS_CMP_LatestTopicsWidget extends BASE_CLASS_Widget
{
    private $entity;
    private $entityId;

    /**
     * @param BASE_CLASS_WidgetParameter $paramObj
     * @return QUESANS_CMP_LatestTopicsWidget
     */
    public function __construct( BASE_CLASS_WidgetParameter $paramObj )
    {
        parent::__construct();

        $confTopicCount = (int) $paramObj->customParamList['topicCount'];

        $this->entityId = (int) $paramObj->additionalParamList['entityId'];
        $this->entity = $paramObj->additionalParamList['entity'];

        $quesansService = QUESANS_BOL_QuesansService::getInstance();
        $quesansGroup = $quesansService->findGroupByEntityId($this->entity, $this->entityId);
        if ( empty($quesansGroup) )
        {
            $this->setVisible(false);
            return;
        }

        $topicList = $quesansService->getGroupTopicList($quesansGroup->getId(), 1, $confTopicCount);
        // get usernames list
        $userIds = array();
        $topicIds = array();

        foreach ( $topicList as $topic )
        {
            array_push($topicIds, $topic['id']);

            if ( isset($topic['lastPost']) && !in_array($topic['lastPost']['userId'], $userIds) )
            {
                array_push($userIds, $topic['lastPost']['userId']);
            }
        }

        $addTopicUrl = OW::getRouter()->urlForRoute('add-topic', array('groupId' => $quesansGroup->getId()));
        $this->assign('addTopicUrl', $addTopicUrl);

        $params = array('entity' => $this->entity, 'entityId' => $this->entityId, 'action' => 'add_topic');
        $event = new OW_Event('quesans.check_permissions', $params);
        OW::getEventManager()->trigger($event);

        $canAdd = $event->getData();

        $this->assign('canAdd', $canAdd);

        $attachments = QUESANS_BOL_PostAttachmentService::getInstance()->getAttachmentsCountByTopicIdList($topicIds);
        $this->assign('attachments', $attachments);

        $usernames = BOL_UserService::getInstance()->getUserNamesForList($userIds);
        $this->assign('usernames', $usernames);

        $displayNames = BOL_UserService::getInstance()->getDisplayNamesForList($userIds);
        $this->assign('displayNames', $displayNames);

        $this->assign('topicList', $topicList);

        if ( $canAdd )
        {
            $this->setSettingValue(
                self::SETTING_TOOLBAR,
                array(
                    array(
                        'label' => OW::getLanguage()->text('quesans', 'add_new'),
                        'href' => OW::getRouter()->urlForRoute('add-topic', array('groupId' => $quesansGroup->getId()))
                    ),
                    array(
                        'label' => OW::getLanguage()->text('base', 'view_all'),
                        'href' => OW::getRouter()->urlForRoute('group-default', array('groupId' => $quesansGroup->getId()))
                    )
                )
            );
        }
        else
        {
            $this->setSettingValue(
                self::SETTING_TOOLBAR,
                array(
                    array(
                        'label' => OW::getLanguage()->text('base', 'view_all'),
                        'href' => OW::getRouter()->urlForRoute('group-default', array('groupId' => $quesansGroup->getId()))
                    )
                )
            );
        }
    }

    public static function getStandardSettingValueList()
    {
        return array(
            self::SETTING_SHOW_TITLE => true,
            self::SETTING_TITLE => OW::getLanguage()->text('quesans', 'widget_latest_topics_label'),
            self::SETTING_WRAP_IN_BOX => false,
            self::SETTING_ICON => self::ICON_FILES
        );
    }

    public static function getSettingList()
    {
        $settingList = array();

        $settingList['topicCount'] = array(
            'presentation' => self::PRESENTATION_NUMBER,
            'label' => OW::getLanguage()->text('quesans', 'cmp_widget_quesans_topics_count'),
            'value' => 3
        );

        return $settingList;
    }

    public static function getAccess()
    {
        return self::ACCESS_ALL;
    }
}