<?php
class QUESANS_CMP_QuesansSearch extends OW_Component
{
    private $scope;
    
    public function __construct( array $params )
    {
        parent::__construct();
        
        $this->scope = $params['scope'];
        
        $value = isset($params['token']) ? trim(htmlspecialchars($params['token'])) : null;
        $userValue = isset($params['userToken']) ? trim(htmlspecialchars($params['userToken'])) : null;
        $invitation = $this->getInvitationLabel();
        
        $inputParams = array(
            'type' => 'text',
            'class' => !mb_strlen($value) ? 'invitation' : '',
            'value' => mb_strlen($value) ? $value : $invitation,
            'id' => UTIL_HtmlTag::generateAutoId('input')
        );
        $this->assign('input', UTIL_HtmlTag::generateTag('input', $inputParams));
        
        $userInputParams = array(
            'type' => 'text',
            'value' => $userValue,
            'id' => $inputParams['id'] . '_user'
        );
        $this->assign('userInput', UTIL_HtmlTag::generateTag('input', $userInputParams));

        $this->addComponent('filterContext', $this->getFilterContextAction());
        $this->assign('themeUrl', OW::getThemeManager()->getCurrentTheme()->getStaticImagesUrl());
        $this->assign('userToken', $userValue);
        
        switch ( $this->scope )
        {
            case 'topic':
                $location = json_encode(OW::getRouter()->urlForRoute('quesans_search_topic', array('topicId' => $params['topicId'])));
                break;
                
            case 'group':
                $location = json_encode(OW::getRouter()->urlForRoute('quesans_search_group', array('groupId' => $params['groupId'])));
                break;

            case 'section':
                $location = json_encode(OW::getRouter()->urlForRoute('quesans_search_section', array('sectionId' => $params['sectionId'])));
                break;

            default:
                $location = json_encode(OW::getRouter()->urlForRoute('quesans_search'));
                break;
        }
        
        $userInvitation = OW::getLanguage()->text('quesans', 'enter_username');

        $script =
        'var invitation = '.json_encode($invitation).';
        var input = '.json_encode($inputParams['id']).';
        var userInvitation = '.json_encode($userInvitation).';
        var userInput = '.json_encode($userInputParams['id']).';

        $("#" + userInput).focus(function() {
            if ( $(this).val() == userInvitation ) {
                $(this).removeClass("invitation").val("");
            }
        });
        $("#" + userInput).blur(function() {
            if ( $(this).val() == "" ) {
                $(this).addClass("invitation").val(userInvitation);
            }
        });
        ';
        
        if ( !mb_strlen($value) )
        {
            $script .=
            '$("#" + input).focus(function() {
                if ( $(this).val() == invitation ) {
                    $(this).removeClass("invitation").val("");
                }
            });
            $("#" + input).blur(function() {
                if ( $(this).val() == "" ) {
                    $(this).addClass("invitation").val(invitation);
                }
            });
            ';
        }

        $script .= 
        'var $form = $("form#quesans_search");
        $(".ow_miniic_delete", $form).click(function() {
            $(".quesans_search_tag_input", $form).css({visibility : "hidden", height: "0px", padding: "0px"});
            $(".add_filter", $form).show();
            $("#quesans_search_cont").removeClass("quesans_search_inputs");
            $("#" + userInput).val("").removeClass("invitation");
        });

        $("#btn-filter-by-user").click(function() {
            $(".quesans_search_tag_input", $form).css({visibility : "visible", height: "auto", padding: "4px"});
            $("#" + userInput).val(userInvitation).addClass("invitation");
            $(".add_filter", $form).hide();
            $("#quesans_search_cont").addClass("quesans_search_inputs");
        });

        $("#'.$inputParams['id'].', #'.$userInputParams['id'].'").keydown(function(e){
            if (e.keyCode == 13) {
                $(this).parents("form").submit();
                return false;
            }
        });
            
        $form.submit(function() {
            var value = $("#" + input).val();
            var userValue = $("#" + userInput).val();

            if ( value == invitation && !userValue.length || userValue == userInvitation && !value.length ) {
                return false;
            }

            if ( value == invitation ) {
                value = ""; $("#" + input).val(value);
            }

            if ( userValue == userInvitation ) {
                userValue = ""; $("#" + userInput).val(userValue);
            }

            var search = encodeURIComponent(value);
            userSearch = encodeURIComponent(userValue);
            document.location.href = '.$location.' + "?"
                + (search.length ? "&q=" + search : "")
                + (userSearch.length ? "&u=" + userSearch : "");

            return false;
        });
        ';

        OW::getDocument()->addOnloadScript($script);
    }
    
    private function getFilterContextAction( )
    {
        $context = new BASE_CMP_ContextAction();

        $action = new BASE_ContextAction();
        $action->setKey('quesans-search');

        $context->addAction($action);

        $action = new BASE_ContextAction();
        $action->setKey('filter_by_user');
        $action->setParentKey('quesans-search');
        $action->setLabel('Filter by user');
        $action->setId('btn-filter-by-user');

        $context->addAction($action);

        return $context;
    }
    
    private function getInvitationLabel()
    {
        return OW::getLanguage()->text('quesans', 'search_invitation_' . $this->scope);
    }
}