<?php
class QUESANS_CMP_QuesansTopicsWidget extends BASE_CLASS_Widget
{
    private $quesansService;

    /**
     * Class constructor
     */
    public function __construct( BASE_CLASS_WidgetParameter $paramObj )
    {
        parent::__construct();

        $this->quesansService = QUESANS_BOL_QuesansService::getInstance();


        $confTopicCount = (int) $paramObj->customParamList['topicCount'];

        $confPostLength = (int) $paramObj->customParamList['postLength'];

        if ( OW::getUser()->isAuthorized('quesans') )
        {
            $excludeGroupIdList = array();
        }
        else
        {
            $excludeGroupIdList = $this->quesansService->getPrivateUnavailableGroupIdList(OW::getUser()->getId());
        }
        $topics = $this->quesansService->getLatestTopicList($confTopicCount, $excludeGroupIdList);

        if ( $topics )
        {
            $this->assign('topics', $topics);

            $userIds = array();
            $groupIds = array();
            $toolbars = array();

            foreach ( $topics as $topic )
            {
                if ( !in_array($topic['lastPost']['userId'], $userIds) )
                {
                    array_push($userIds, $topic['lastPost']['userId']);
                }

                if ( !in_array($topic['groupId'], $groupIds) )
                {
                    array_push($groupIds, $topic['groupId']);
                }
            }

            $avatars = BOL_AvatarService::getInstance()->getDataForUserAvatars($userIds, true, false);
            $this->assign('avatars', $avatars);

            $urls = BOL_UserService::getInstance()->getUserUrlsForList($userIds);

            // toolbars
            foreach ( $topics as $key => $topic )
            {
                $userId = $topic['lastPost']['userId'];
                $toolbars[$topic['lastPost']['postId']][] = array(
                    'class' => 'ow_icon_control ow_ic_user',
                    'href' => !empty($urls[$userId]) ? $urls[$userId] : '#',
                    'label' => !empty($avatars[$userId]['title']) ? $avatars[$userId]['title'] : ''
                );

                $toolbars[$topic['lastPost']['postId']][] = array(
                    'label' => $topic['lastPost']['createStamp'],
                    'class' => 'ow_ipc_date'
                );
            }
            $this->assign('toolbars', $toolbars);

            $this->assign('postLength', $confPostLength);

            $groups = $this->quesansService->findGroupByIdList($groupIds);

            $groupList = array();

            $sectionIds = array();

            foreach ( $groups as $group )
            {
                $groupList[$group->id] = $group;

                if ( !in_array($group->sectionId, $sectionIds) )
                {
                    array_push($sectionIds, $group->sectionId);
                }
            }
            $this->assign('groups', $groupList);

            $sectionList = $this->quesansService->findSectionsByIdList($sectionIds);
            $this->assign('sections', $sectionList);

            $tb = array();
            if ( OW::getUser()->isAuthorized('quesans', 'edit') )
            {
                $tb[] = array(
                    'label' => OW::getLanguage()->text('quesans', 'add_new'),
                    'href' => OW::getRouter()->urlForRoute('add-topic-default')
                );
            }
            $tb[] = array(
                'label' => OW::getLanguage()->text('quesans', 'goto_quesans'),
                'href' => OW::getRouter()->urlForRoute('quesans-default')
            );

            $this->setSettingValue(self::SETTING_TOOLBAR, $tb);
        }
        else
        {
            if ( !OW::getUser()->isAuthorized('quesans', 'edit') )
            {
                $this->setVisible(false);

                return;
            }

            $this->assign('topics', null);
        }
    }

    public static function getSettingList()
    {
        $settingList = array();

        $settingList['topicCount'] = array(
            'presentation' => self::PRESENTATION_NUMBER,
            'label' => OW::getLanguage()->text('quesans', 'cmp_widget_quesans_topics_count'),
            'value' => 5
        );

        $settingList['postLength'] = array(
            'presentation' => self::PRESENTATION_NUMBER,
            'label' => OW::getLanguage()->text('quesans', 'cmp_widget_quesans_topics_post_lenght'),
            'value' => 50
        );

        return $settingList;
    }

    public static function validateSettingList( $settingList )
    {
        parent::validateSettingList($settingList);

        $validationMessage = OW::getLanguage()->text('quesans', 'cmp_widget_quesans_topics_count_msg');

        if ( !preg_match('/^\d+$/', $settingList['topicCount']) )
        {
            throw new WidgetSettingValidateException($validationMessage, 'topicCount');
        }
        if ( $settingList['topicCount'] > 20 )
        {
            throw new WidgetSettingValidateException($validationMessage, 'topicCount');
        }

        $validationMessage = OW::getLanguage()->text('quesans', 'cmp_widget_quesans_topics_post_length_msg');

        if ( !preg_match('/^\d+$/', $settingList['postLength']) )
        {
            throw new WidgetSettingValidateException($validationMessage, 'postLength');
        }
        if ( $settingList['postLength'] > 1000 )
        {
            throw new WidgetSettingValidateException($validationMessage, 'postLength');
        }
    }

    public static function getAccess()
    {
        return self::ACCESS_ALL;
    }

    public static function getStandardSettingValueList()
    {
        return array(
            self::SETTING_TITLE => OW::getLanguage()->text('quesans', 'quesans_topics_widget'),
            self::SETTING_ICON => self::ICON_FILES,
            self::SETTING_SHOW_TITLE => true
        );
    }
}