<?php
class QUESANS_CTRL_AddTopic extends OW_ActionController
{

    /**
     * Controller's default action
     *
     * @param array $params
     * @throws AuthorizationException
     * @throws AuthenticateException
     */
    public function index( array $params = null )
    {
        $groupId = isset($params['groupId']) && (int) $params['groupId'] ? (int) $params['groupId'] : 0;

        $quesansService = QUESANS_BOL_QuesansService::getInstance();

        $quesansGroup = $quesansService->getGroupInfo($groupId);
        if ( $quesansGroup )
        {
            $quesansSection = $quesansService->findSectionById($quesansGroup->sectionId);
            $isHidden = $quesansSection->isHidden;
        }
        else
        {
            $isHidden = false;
        }

        if ( !OW::getUser()->isAuthenticated() )
        {
            throw new AuthenticateException();
        }

        $userId = OW::getUser()->getId();

        $this->assign('authMsg', null);

        if ( $isHidden && isset($quesansSection) )
        {
            $eventParams = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'add_topic');
            $event = new OW_Event('quesans.check_permissions', $eventParams);
            OW::getEventManager()->trigger($event);

            if ( !$event->getData() )
            {
                throw new AuthorizationException();
            }

            if ( !OW::getUser()->isAuthorized($quesansSection->entity, 'add_topic') )
            {
                $status = BOL_AuthorizationService::getInstance()->getActionStatus($quesansSection->entity, 'add_topic');
                throw new AuthorizationException($status['msg']);
            }

            $event = new OW_Event('quesans.find_quesans_caption', array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId));
            OW::getEventManager()->trigger($event);

            $eventData = $event->getData();

            /** @var OW_Component $componentQuesansCaption */
            $componentQuesansCaption = $eventData['component'];

            if ( !empty($componentQuesansCaption) )
            {
                $this->assign('componentQuesansCaption', $componentQuesansCaption->render());
            }
            else
            {
                $componentQuesansCaption = false;
                $this->assign('componentQuesansCaption', $componentQuesansCaption);
            }

            $bcItems = array(
                array(
                    'href' => OW::getRouter()->urlForRoute('group-default', array('groupId' => $quesansGroup->getId())),
                    'label' => OW::getLanguage()->text($quesansSection->entity, 'view_all_topics')
                )
            );

            $breadCrumbCmp = new BASE_CMP_Breadcrumb($bcItems);
            $this->addComponent('breadcrumb', $breadCrumbCmp);

            OW::getNavigation()->deactivateMenuItems(OW_Navigation::MAIN);
            OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, $quesansSection->entity, $eventData['key']);

            $groupSelect = array(array('label' => $quesansGroup->name, 'value' => $quesansGroup->getId(), 'disabled' => false));

            OW::getDocument()->setHeading(OW::getLanguage()->text($quesansSection->entity, 'create_new_topic', array('group' => $quesansGroup->name)));
        }
        else
        {
            $canEdit = OW::getUser()->isAuthorized('quesans', 'edit');

            if ( !$userId )
            {
                throw new AuthorizationException();
            }
            else if ( !$canEdit )
            {
                $status = BOL_AuthorizationService::getInstance()->getActionStatus('quesans', 'edit');
                throw new AuthorizationException($status['msg']);
            }

            if ( !OW::getRequest()->isAjax() )
            {
                OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
            }

            $groupSelect = $quesansService->getGroupSelectList(0, false, $userId);

            OW::getDocument()->setHeading(OW::getLanguage()->text('quesans', 'create_new_topic'));
        }

        OW::getDocument()->setDescription(OW::getLanguage()->text('quesans', 'meta_description_add_topic'));
        OW::getDocument()->setTitle(OW::getLanguage()->text('quesans', 'meta_title_add_topic'));
        OW::getDocument()->setHeadingIconClass('ow_ic_write');

        $this->assign('isHidden', $isHidden);

        $uid = uniqid();
        $form = $this->generateForm($groupSelect, $groupId, $isHidden, $uid);

        OW::getDocument()->addStyleDeclaration('
			.disabled_option {
				color: #9F9F9F;
    		}
		');

        $enableAttachments = OW::getConfig()->getValue('quesans', 'enable_attachments');
        if ( $enableAttachments )
        {
            $attachmentCmp = new BASE_CLASS_FileAttachment('quesans', $uid);
            $this->addComponent('attachments', $attachmentCmp);
        }

        $this->assign('enableAttachments', $enableAttachments);

        if ( OW::getRequest()->isPost() && $form->isValid($_POST) )
        {
            $data = $form->getValues();

            if ( $data['group'] )
            {
                $topicDto = new QUESANS_BOL_Topic();

                $topicDto->userId = $userId;
                $topicDto->groupId = $data['group'];
                $topicDto->title = strip_tags($data['title']);

                $quesansService->addTopic($topicDto);

                $postDto = new QUESANS_BOL_Post();

                $postDto->topicId = $topicDto->id;
                $postDto->userId = $userId;

                $postDto->text = UTIL_HtmlTag::stripJs(UTIL_HtmlTag::stripTags($data['text'], array('form', 'input', 'button'), null, true));
                $postDto->createStamp = time();

                $quesansService->saveOrUpdatePost($postDto);
                $topicDto->lastPostId = $postDto->getId();

                $quesansService->saveOrUpdateTopic($topicDto);

                // subscribe author to new posts
                if ( $data['subscribe'] )
                {
                    $subService = QUESANS_BOL_SubscriptionService::getInstance();

                    $subs = new QUESANS_BOL_Subscription();
                    $subs->userId = $userId;
                    $subs->topicId = $topicDto->id;

                    $subService->addSubscription($subs);
                }

                if ( $enableAttachments )
                {
                    $filesArray = BOL_AttachmentService::getInstance()->getFilesByBundleName('quesans', $data['attachmentUid']);

                    if ( $filesArray )
                    {
                        $attachmentService = QUESANS_BOL_PostAttachmentService::getInstance();
                        $skipped = 0;

                        foreach ( $filesArray as $file )
                        {
                            $attachmentDto = new QUESANS_BOL_PostAttachment();
                            $attachmentDto->postId = $postDto->id;
                            $attachmentDto->fileName = $file['dto']->origFileName;
                            $attachmentDto->fileNameClean = $file['dto']->fileName;
                            $attachmentDto->fileSize = $file['dto']->size * 1024;
                            $attachmentDto->hash = uniqid();

                            $added = $attachmentService->addAttachment($attachmentDto, $file['path']);

                            if ( !$added )
                            {
                                $skipped++;
                            }
                        }
                        
                        BOL_AttachmentService::getInstance()->deleteAttachmentByBundle('quesans', $data['attachmentUid']);

                        if ( $skipped )
                        {
                            OW::getFeedback()->warning(OW::getLanguage()->text('quesans', 'not_all_attachments_added'));
                        }
                    }
                }

                $topicUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicDto->id));

                //Newsfeed
                $params = array(
                    'pluginKey' => 'quesans',
                    'entityType' => 'quesans-topic',
                    'entityId' => $topicDto->id,
                    'userId' => $topicDto->userId
                );

                $event = new OW_Event('feed.action', $params);
                OW::getEventManager()->trigger($event);

                if ( $isHidden && isset($quesansSection) )
                {
                    BOL_AuthorizationService::getInstance()->trackAction($quesansSection->entity, 'add_topic');
                }
                else
                {
                    BOL_AuthorizationService::getInstance()->trackAction('quesans', 'edit');
                }
                
                    $params = array(
                        'topicId' => $topicDto->id,
                    'entity' => $quesansSection->entity ? $quesansSection->entity : NULL,
                    'entityId' => $quesansGroup->entityId ? $quesansGroup->entityId : NULL,
                        'userId' => $topicDto->userId,
                        'topicUrl' => $topicUrl,
                        'topicTitle' => $topicDto->title,
                        'postText' => $postDto->text
                    );
                    $event = new OW_Event('quesans.topic_add', $params);
                    OW::getEventManager()->trigger($event);

                OW::getEventManager()->trigger(new OW_Event(QUESANS_BOL_QuesansService::EVENT_AFTER_TOPIC_ADD, array(
                    'topicId' => $topicDto->id
                )));

                $this->redirect($topicUrl);
            }
            else
            {
                $form->getElement('group')->addError(OW::getLanguage()->text('quesans', 'select_group_error'));
            }
        }
    }

    /**
     * Generates Add Topic Form.
     *
     * @param array $groupSelect
     * @param int $groupId
     * @param $isHidden
     * @param $uid
     * @return Form
     */
    private function generateForm( $groupSelect, $groupId, $isHidden, $uid )
    {
        $form = new Form('add-topic-form');
        $form->setEnctype("multipart/form-data");

        $lang = OW::getLanguage();

        $attachmentUid = new HiddenField('attachmentUid');
        $attachmentUid->setValue($uid);
        $attachmentUid->setRequired(true);
        $form->addElement($attachmentUid);

        $title = new TextField('title');
        $title->setRequired(true);
        $sValidator = new StringValidator(1, 255);
        $sValidator->setErrorMessage($lang->text('quesans', 'chars_limit_exceeded', array('limit' => 255)));
        $title->addValidator($sValidator);
        $form->addElement($title);

        if ( $isHidden )
        {
            $group = new HiddenField('group');
            $group->setValue($groupId);
        }
        else
        {
            $group = new QuesansSelectBox('group');
            $group->setOptions($groupSelect);
            if ( $groupId )
            {
                $group->setValue($groupId);
            }
            $group->setRequired(true);
            $group->addValidator(new IntValidator());
        }

        $form->addElement($group);

        $btnSet = array(BOL_TextFormatService::WS_BTN_IMAGE, BOL_TextFormatService::WS_BTN_VIDEO, BOL_TextFormatService::WS_BTN_HTML);
        $text = new WysiwygTextarea('text', $btnSet);
        $text->setRequired(true);
        $sValidator = new StringValidator(1, 50000);
        $sValidator->setErrorMessage($lang->text('quesans', 'chars_limit_exceeded', array('limit' => 50000)));
        $text->addValidator($sValidator);
        $form->addElement($text);

        $subscribe = new CheckboxField('subscribe');
        $subscribe->setLabel($lang->text('quesans', 'subscribe'));
        $subscribe->setValue(true);
        $form->addElement($subscribe);

        $post = new Submit('post');
        $post->setValue($lang->text('quesans', 'add_post_btn'));
        $form->addElement($post);

        $this->addForm($form);

        return $form;
    }
}
