<?php
class QUESANS_CTRL_Customize extends OW_ActionController
{

    /**
     * Controller's default action
     */
    public function index()
    {
        $quesansService = QUESANS_BOL_QuesansService::getInstance();

        $sectionGroupList = $quesansService->getCustomSectionGroupList();
        $isModerator = OW::getUser()->isAuthorized('quesans');

        if ( !$isModerator )
        {
            throw new Redirect404Exception();
        }

        $this->assign('sectionGroupList', $sectionGroupList);
        
        $plugin = OW::getPluginManager()->getPlugin('quesans');

        //add langs for javascript
        OW::getLanguage()->addKeyForJs('quesans', 'delete_section_confirm');
        OW::getLanguage()->addKeyForJs('quesans', 'delete_group_confirm');
        OW::getLanguage()->addKeyForJs('quesans', 'add_new_quesans_title');
        OW::getLanguage()->addKeyForJs('quesans', 'edit_section_title');
        OW::getLanguage()->addKeyForJs('quesans', 'edit_group_title');
        
        OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('base')->getStaticJsUrl() . 'jquery-ui.min.js');
        OW::getDocument()->addScript($plugin->getStaticJsUrl() . 'quesans.js');

        $sortSectionOrderUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'sortSectionOrder');
        $sortGroupOrderUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'sortGroupOrder');
        $deleteSectionUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'deleteSection');
        $deleteGroupUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'deleteGroup');
        $getSectionUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'getSection');
        $getGroupUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'getGroup');
        $editSectionUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'editSection');
        $editGroupUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'editGroup');
        $addQuesansUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'addQuesans');
        $suggestSectionUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'suggestSection');
        $quesansIndexUrl = OW::getRouter()->urlForRoute('quesans-default');

        $onloadJs = "
			QuesansCustomize.sortSectionOrderUrl = '$sortSectionOrderUrl';
			QuesansCustomize.sortGroupOrderUrl = '$sortGroupOrderUrl';
			QuesansCustomize.deleteSectionUrl = '$deleteSectionUrl';
			QuesansCustomize.deleteGroupUrl = '$deleteGroupUrl';
			QuesansCustomize.suggestSectionUrl = '$suggestSectionUrl';
			QuesansCustomize.getSectionUrl = '$getSectionUrl';
			QuesansCustomize.getGroupUrl = '$getGroupUrl';
			QuesansCustomize.quesansIndexUrl = '$quesansIndexUrl';
			
        	QuesansCustomize.construct();
        	";
        OW::getDocument()->addOnloadScript($onloadJs);

        $addQuesansForm = $this->generateAddQuesansForm($addQuesansUrl);
        $this->addForm($addQuesansForm);

        $editSectionForm = $this->generateEditSectionForm($editSectionUrl);
        $this->addForm($editSectionForm);

        $editGroupForm = $this->generateEditGroupForm($editGroupUrl);
        $this->addForm($editGroupForm);

        OW::getDocument()->setHeading(OW::getLanguage()->text('quesans', 'quesans'));
        OW::getDocument()->setHeadingIconClass('ow_ic_quesans');
        
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
    }

    /**
     * This action updates section orders called by ajax request
     */
    public function sortSectionOrder()
    {
        $isModerator = OW::getUser()->isAuthorized('quesans');

        if ( OW::getRequest()->isAjax() && $_POST['data'] && $isModerator )
        {
            $quesansService = QUESANS_BOL_QuesansService::getInstance();

            $postData = array();
            parse_str($_POST['data'], $postData);

            if ( !$postData['section'] )
            {
                return false;
            }

            $sectionOrderList = array_flip($postData['section']);

            $sectionDtoList = $quesansService->findSectionList();

            foreach ( $sectionDtoList as $sectionDto )
            {
                $sectionDto->order = $sectionOrderList[$sectionDto->id] + 1;
                $quesansService->saveOrUpdateSection($sectionDto);
            }
        }

        exit();
    }

    /**
     * This action updates group orders called by ajax request
     */
    public function sortGroupOrder()
    {
        $isModerator = OW::getUser()->isAuthorized('quesans');

        if ( OW::getRequest()->isAjax() && $_POST['data'] && $isModerator )
        {
            $quesansService = QUESANS_BOL_QuesansService::getInstance();

            $sectionGroupList = json_decode($_POST['data'], true);

            foreach ( $sectionGroupList as $sectionGroup )
            {
                if ( !$sectionGroup['order'] )
                {
                    continue;
                }

                $sectionId = (int) $sectionGroup['sectionId'];

                $postData = array();
                parse_str($sectionGroup['order'], $postData);

                $groupOrderList = array_flip($postData['group']);

                $groupDtoList = $quesansService->findGroupByIdList($postData['group']);
                $groupDto = new QUESANS_BOL_Group();
                foreach ( $groupDtoList as $groupDto )
                {
                    $groupDto->sectionId = $sectionId;
                    $groupDto->order = $groupOrderList[$groupDto->id] + 1;

                    $quesansService->saveOrUpdateGroup($groupDto);
                }
            }
        }

        exit();
    }

    /**
     * This action deletes section called by ajax request
     */
    public function deleteSection()
    {
        $isModerator = OW::getUser()->isAuthorized('quesans');

        $sectionId = (int) $_POST['sectionId'];

        if ( OW::getRequest()->isAjax() && $sectionId && $isModerator )
        {
            $quesansService = QUESANS_BOL_QuesansService::getInstance();

            $quesansService->deleteSection($sectionId);

            echo 1;
        }

        exit();
    }

    /**
     * This action deletes group called by ajax request
     */
    public function deleteGroup()
    {
        $isModerator = OW::getUser()->isAuthorized('quesans');

        $groupId = (int) $_POST['groupId'];

        if ( OW::getRequest()->isAjax() && $groupId && $isModerator )
        {
            $quesansService = QUESANS_BOL_QuesansService::getInstance();

            $quesansService->deleteGroup($groupId);

            echo 1;
        }

        exit();
    }

    /**
     * This action adds section and group called by ajax request
     */
    public function addQuesans()
    {
        $isModerator = OW::getUser()->isAuthorized('quesans');

        if ( OW::getRequest()->isAjax() && $_POST && $isModerator )
        {
            $groupName = trim($_POST['group-name']);
            $sectionName = trim($_POST['section']);
            $groupDescription = trim($_POST['description']);
            $isPrivate = $_POST['is-private'] == 'on';
            $roles = !empty($_POST['roles']) ? $_POST['roles'] : null;

            if ( !$groupName || !$sectionName || !$groupDescription )
            {
                exit();
            }

            $quesansService = QUESANS_BOL_QuesansService::getInstance();

            $sectionDto = $quesansService->getPublicSection($sectionName, 0);

            //create quesans section
            if ( $sectionDto === null )
            {
                $sectionDto = new QUESANS_BOL_Section();
                $sectionDto->name = $sectionName;
                $sectionDto->order = $quesansService->getNewSectionOrder();

                $quesansService->saveOrUpdateSection($sectionDto);
            }

            //create quesans group
            $groupDto = new QUESANS_BOL_Group();

            $groupDto->sectionId = $sectionDto->id;
            $groupDto->name = $groupName;
            $groupDto->description = $groupDescription;
            $groupDto->order = $quesansService->getNewGroupOrder($sectionDto->id);
            $groupDto->isPrivate = (bool) $isPrivate;
            $groupDto->roles = count($roles) ? json_encode($roles) : null;

            $quesansService->saveOrUpdateGroup($groupDto);

            echo true;
            exit();
        }
    }

    /**
     * This action suggests sections called by ajax request
     */
    public function suggestSection()
    {
        if ( OW::getRequest()->isAjax() && isset($_GET['q']) )
        {
            $sectionName = trim($_GET['q']);

            $quesansService = QUESANS_BOL_QuesansService::getInstance();
            $sectionDtoList = $quesansService->suggestSection($sectionName);

            if ( $sectionDtoList )
            {
                foreach ( $sectionDtoList as $sectionDto )
                {
                    echo "$sectionDto->name\t$sectionDto->id\n";
                }
            }
        }

        exit();
    }

    /**
     * This action edits section called by ajax request
     */
    public function editSection()
    {
        $isModerator = OW::getUser()->isAuthorized('quesans');

        if ( OW::getRequest()->isAjax() && $_POST && $isModerator )
        {
            $sectionName = trim($_POST['section-name']);
            $sectionId = (int) $_POST['section-id'];

            if ( !$sectionName || !$sectionId )
            {
                exit();
            }

            $quesansService = QUESANS_BOL_QuesansService::getInstance();
            $sectionDto = $quesansService->findSectionById($sectionId);

            if ( $sectionDto === null )
            {
                exit();
            }

            $sectionDto->name = $sectionName;

            $quesansService->saveOrUpdateSection($sectionDto);

            echo true;
            exit();
        }
    }

    /**
     * This action edits group called by ajax request
     */
    public function editGroup()
    {
        $isModerator = OW::getUser()->isAuthorized('quesans');

        if ( OW::getRequest()->isAjax() && $_POST && $isModerator )
        {
            $groupId = (int) $_POST['group-id'];
            $groupName = trim($_POST['group-name']);
            $groupDescription = trim($_POST['description']);
            $isPrivate = $_POST['is-private'] == 'on';
            $roles = !empty($_POST['roles']) ? $_POST['roles'] : null;

            if ( !$groupId || !$groupName || !$groupDescription )
            {
                exit();
            }

            $quesansService = QUESANS_BOL_QuesansService::getInstance();
            $groupDto = $quesansService->findGroupById($groupId);

            if ( $groupDto === null )
            {
                exit();
            }

            $groupDto->name = $groupName;
            $groupDto->description = $groupDescription;
            $groupDto->isPrivate = (bool) $isPrivate;
            $groupDto->roles = count($roles) ? json_encode($roles) : null;

            $quesansService->saveOrUpdateGroup($groupDto);

            echo true;
            exit();
        }
    }

    /**
     * This action returns section called by ajax request
     */
    public function getSection()
    {
        if ( OW::getRequest()->isAjax() && $_POST['sectionId'] )
        {
            $sectionId = (int) $_POST['sectionId'];

            $quesansService = QUESANS_BOL_QuesansService::getInstance();
            $sectionDto = $quesansService->findSectionById($sectionId);

            echo json_encode($sectionDto);
        }

        exit();
    }

    /**
     * This action returns group called by ajax request
     */
    public function getGroup()
    {
        if ( OW::getRequest()->isAjax() && $_POST['groupId'] )
        {
            $groupId = (int) $_POST['groupId'];

            $quesansService = QUESANS_BOL_QuesansService::getInstance();
            $groupDto = $quesansService->findGroupById($groupId);

            $groupDto->roles = json_decode($groupDto->roles, true);
            echo json_encode($groupDto);
        }

        exit();
    }

    /**
     * Generates add quesans form
     * 
     * @param string $action
     * @return Form
     */
    private function generateAddQuesansForm( $action )
    {
        $language = OW::getLanguage();
        $form = new Form('add-quesans-form');

        $form->setAction($action);

        $groupName = new TextField('group-name');
        $groupName->setRequired(true);
        $sValidator = new StringValidator(1, 255);
        $sValidator->setErrorMessage($language->text('quesans', 'chars_limit_exceeded', array('limit' => 255)));
        $groupName->addValidator($sValidator);
        $form->addElement($groupName);

        $sectionField = new SuggestField('section');
        $sectionField->setRequired(true);
        $sectionField->setMinChars(1);
        $responderUrl = OW::getRouter()->urlFor('QUESANS_CTRL_Customize', 'suggestSection');
        $sectionField->setResponderUrl($responderUrl);

        $firstSection = QUESANS_BOL_QuesansService::getInstance()->getFirstSection();
        if ( $firstSection )
        {
            $sectionField->setValue($firstSection->name);
        }

        $form->addElement($sectionField->setLabel($language->text('quesans', 'section')));

        $description = new Textarea('description');
        $description->setRequired(true);
        $sValidator = new StringValidator(1, 50000);
        $sValidator->setErrorMessage($language->text('quesans', 'chars_limit_exceeded', array('limit' => 50000)));
        $description->addValidator($sValidator);
        $form->addElement($description);

        $isPrivate = new CheckboxField('is-private');
        $form->addElement($isPrivate);
        
        $roles = new CheckboxGroup('roles');
        $authService = BOL_AuthorizationService::getInstance();
        $roleList = $authService->getRoleList();
        $options = array();
        foreach ( $roleList as $role )
        {
            $options[$role->id] = $authService->getRoleLabel($role->name);
        }
        $roles->addOptions($options);
        $roles->setColumnCount(2);
        $form->addElement($roles);
        
        $submit = new Submit('add');
        $submit->setValue($language->text('quesans', 'add_new_quesans_submit'));
        $form->addElement($submit);

        $form->setAjax(true);

        return $form;
    }

    /**
     * Generates edit section form
     * 
     * @param string $action
     * @return Form
     */
    private function generateEditSectionForm( $action )
    {
        $form = new Form('edit-section-form');
        $form->setAction($action);
        
        $lang = OW::getLanguage();

        $sectionName = new TextField('section-name');
        $sectionName->setRequired(true);
        $sValidator = new StringValidator(1, 255);
        $sValidator->setErrorMessage($lang->text('quesans', 'chars_limit_exceeded', array('limit' => 255)));
        $sectionName->addValidator($sValidator);
        $form->addElement($sectionName);

        $sectionId = new HiddenField('section-id');
        $sectionId->setRequired(true);
        $form->addElement($sectionId);

        $submit = new Submit('save');
        $submit->setValue($lang->text('quesans', 'edit_section_btn'));
        $form->addElement($submit);

        $form->setAjax(true);

        return $form;
    }

    /**
     * Generates edit group form
     * 
     * @param string $action
     * @return Form
     */
    private function generateEditGroupForm( $action )
    {
        $form = new Form('edit-group-form');
        $form->setAction($action);
        
        $lang = OW::getLanguage();

        $groupName = new TextField('group-name');
        $groupName->setRequired(true);
        $sValidator = new StringValidator(1, 255);
        $sValidator->setErrorMessage($lang->text('quesans', 'chars_limit_exceeded', array('limit' => 255)));
        $groupName->addValidator($sValidator);
        $form->addElement($groupName);

        $description = new Textarea('description');
        $description->setRequired(true);
        $sValidator = new StringValidator(1, 50000);
        $sValidator->setErrorMessage($lang->text('quesans', 'chars_limit_exceeded', array('limit' => 50000)));
        $description->addValidator($sValidator);
        $form->addElement($description);

        $groupId = new HiddenField('group-id');
        $groupId->setRequired(true);
        $form->addElement($groupId);
        
        $isPrivate = new CheckboxField('is-private');
        $form->addElement($isPrivate);
        
        $roles = new CheckboxGroup('roles');
        $authService = BOL_AuthorizationService::getInstance();
        $roleList = $authService->getRoleList();
        $options = array();
        foreach ( $roleList as $role )
        {
            $options[$role->id] = $authService->getRoleLabel($role->name);
        }
        $roles->addOptions($options);
        $roles->setColumnCount(2);
        $form->addElement($roles);

        $submit = new Submit('save');
        $submit->setValue($lang->text('quesans', 'edit_group_btn'));
        $form->addElement($submit);

        $form->setAjax(true);

        return $form;
    }
}