<?php
class QUESANS_CTRL_EditPost extends OW_ActionController
{

    public function index( array $params = null )
    {
	        $quesansService = QUESANS_BOL_QuesansService::getInstance();

        if ( !isset($params['id']) || !($postId = (int) $params['id']) )
        {
            throw new Redirect404Exception();
        }

        $postDto = $quesansService->findPostById($postId);

        if ( !$postDto )
        {
            throw new Redirect404Exception();
        }

        $userId = OW::getUser()->getId();
        $topicId = $postDto->topicId;
        $topicDto = $quesansService->findTopicById($topicId);

        $quesansGroup = $quesansService->getGroupInfo($topicDto->groupId);
        $quesansSection = $quesansService->findSectionById($quesansGroup->sectionId);

        $isHidden = $quesansSection->isHidden;

        if ( $isHidden )
        {
            $isModerator = OW::getUser()->isAuthorized($quesansSection->entity);

            $eventParams = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'add_topic');
            $event = new OW_Event('quesans.check_permissions', $eventParams);
            OW::getEventManager()->trigger($event);
            $canPost = $event->getData();

            //check permissions
            $canEdit = OW::getUser()->isAuthorized($quesansSection->entity, 'add_topic') && $userId == $postDto->userId;

            if ( !$isModerator )
            {
                if ( !$canPost )
                {
                    throw new AuthorizationException();
                }
                else if ( !$canEdit )
                {
                    $status = BOL_AuthorizationService::getInstance()->getActionStatus(
                        $quesansSection->entity, 'add_topic'
                    );
                    throw new AuthorizationException($status['msg']);
                }
            }
        }
        else
        {
            $isModerator = OW::getUser()->isAuthorized('quesans');
            $canEdit = $postDto->userId == OW::getUser()->getId();

            if ( !$canEdit && !$isModerator )
            {
                throw new AuthorizationException();
            }
        }

        $this->assign('postId', $postId);

        $uid = uniqid();
        $editPostForm = $this->generateEditPostForm($postDto, $uid);
        $this->addForm($editPostForm);

        $lang = OW::getLanguage();
        $router = OW::getRouter();

        $enableAttachments = OW::getConfig()->getValue('quesans', 'enable_attachments');
        $this->assign('enableAttachments', $enableAttachments);

        if ( $enableAttachments )
        {
            $attachments = QUESANS_BOL_PostAttachmentService::getInstance()->findAttachmentsByPostIdList(array($postId));
            $this->assign('attachments', $attachments);

            $attachmentCmp = new BASE_CLASS_FileAttachment('quesans', $uid);
            $this->addComponent('attachmentsCmp', $attachmentCmp);
        }

        if ( OW::getRequest()->isPost() && $editPostForm->isValid($_POST) )
        {
            $values = $editPostForm->getValues();
            
            $postId = (int) $values['post-id'];
            $text = $values['text'];
            $topicId = (int) $values['topic'];

            $topicUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicId));

            $postDto = $quesansService->findPostById($postId);

            if ( $postDto === null || ($postDto->userId != $userId && !$isModerator) )
            {
                $this->redirect($topicUrl);
            }

            //save post
            $postDto->text = UTIL_HtmlTag::stripJs(UTIL_HtmlTag::stripTags($text, array('form', 'input', 'button'), null, true));
            $quesansService->saveOrUpdatePost($postDto);

            //save post edit info
            $editPostDto = $quesansService->findEditPost($postId);

            if ( $editPostDto === null )
            {
                $editPostDto = new QUESANS_BOL_EditPost();
            }

            $editPostDto->postId = $postId;
            $editPostDto->userId = $userId;
            $editPostDto->editStamp = time();

            $quesansService->saveOrUpdateEditPost($editPostDto);

            if ( $enableAttachments )
            {
                $filesArray = BOL_AttachmentService::getInstance()->getFilesByBundleName('quesans', $values['attachmentUid']);

                if ( $filesArray )
                {
                    $attachmentService = QUESANS_BOL_PostAttachmentService::getInstance();
                    $skipped = 0;

                    foreach ( $filesArray as $file )
                    {
                        $attachmentDto = new QUESANS_BOL_PostAttachment();
                        $attachmentDto->postId = $postDto->id;
                        $attachmentDto->fileName = $file['dto']->origFileName;
                        $attachmentDto->fileNameClean = $file['dto']->fileName;
                        $attachmentDto->fileSize = $file['dto']->size * 1024;
                        $attachmentDto->hash = uniqid();

                        $added = $attachmentService->addAttachment($attachmentDto, $file['path']);

                        if ( !$added )
                        {
                            $skipped++;
                        }
                    }

                    BOL_AttachmentService::getInstance()->deleteAttachmentByBundle('quesans', $values['attachmentUid']);
                    
                    if ( $skipped )
                    {
                        OW::getFeedback()->warning(OW::getLanguage()->text('quesans', 'not_all_attachments_added'));
                    }
                }
            }

            $this->redirect($quesansService->getPostUrl($topicId, $postId, true));
        }

        $topicInfo = $quesansService->getTopicInfo($topicId);
        $groupUrl = $router->urlForRoute('group-default', array('groupId' => $topicDto->groupId));
        $topicUrl = $quesansService->getPostUrl($topicId, $postId);

        $lang->addKeyForJs('quesans', 'confirm_delete_attachment');

        OW::getDocument()->setHeading(OW::getLanguage()->text('quesans', 'edit_post_title'));
        OW::getDocument()->setHeadingIconClass('ow_ic_edit');

        $this->assign('isHidden', $isHidden);

        if ( $isHidden )
        {
            $event = new OW_Event('quesans.find_quesans_caption', array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId));
            OW::getEventManager()->trigger($event);

            $eventData = $event->getData();

            /** @var OW_Component $componentQuesansCaption */
            $componentQuesansCaption = $eventData['component'];

            if (!empty($componentQuesansCaption))
            {
                $this->assign('componentQuesansCaption', $componentQuesansCaption->render());
            }
            else
            {
                $componentQuesansCaption = false;
                $this->assign('componentQuesansCaption', $componentQuesansCaption);
            }

            $bcItems = array(
                array(
                    'href' => OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicId)),
                    'label' => OW::getLanguage()->text('quesans', 'back_to_topic')
                )
            );

            $breadCrumbCmp = new BASE_CMP_Breadcrumb($bcItems);
            $this->addComponent('breadcrumb', $breadCrumbCmp);

            OW::getNavigation()->deactivateMenuItems(OW_Navigation::MAIN);
            OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, $quesansSection->entity, $eventData['key']);
        }
        else
        {
            $bcItems = array(
                array(
                    'href' => $router->urlForRoute('quesans-default'),
                    'label' => $lang->text('quesans', 'quesans_index')
                ),
                array(
                    'href' => $router->urlForRoute('quesans-default') . '#section-' . $topicInfo['sectionId'],
                    'label' => $topicInfo['sectionName']
                ),
                array(
                    'href' => $groupUrl,
                    'label' => $topicInfo['groupName']
                ),
                array(
                    'href' => $topicUrl,
                    'label' => htmlspecialchars($topicDto->title)
                )
            );

            $breadCrumbCmp = new BASE_CMP_Breadcrumb($bcItems, $lang->text('quesans', 'post_location'));
            $this->addComponent('breadcrumb', $breadCrumbCmp);

            OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
        }
    }

    /**
     * Generates edit post form.
     *
     * @param $postDto
     * @param $uid
     * @return Form
     */
    private function generateEditPostForm( $postDto, $uid )
    {
        $form = new Form('edit-post-form');
        $form->setEnctype('multipart/form-data');
        
        $lang = OW::getLanguage();

        $postIdField = new HiddenField('post-id');
        $postIdField->setValue($postDto->id);
        $form->addElement($postIdField);

        $attachmentUid = new HiddenField('attachmentUid');
        $attachmentUid->setValue($uid);
        $attachmentUid->setRequired(true);
        $form->addElement($attachmentUid);

        $topicIdField = new HiddenField('topic');
        $topicIdField->setValue($postDto->topicId);
        $form->addElement($topicIdField);

        $btnSet = array(BOL_TextFormatService::WS_BTN_IMAGE, BOL_TextFormatService::WS_BTN_VIDEO, BOL_TextFormatService::WS_BTN_HTML);
        $postText = new WysiwygTextarea('text', $btnSet);
        $postText->setRequired(true);
        $postText->setValue($postDto->text);
        $sValidator = new StringValidator(1, 50000);
        $sValidator->setErrorMessage($lang->text('quesans', 'chars_limit_exceeded', array('limit' => 50000)));
        $postText->addValidator($sValidator);
        $form->addElement($postText);

        $submit = new Submit('save');
        $submit->setValue($lang->text('base', 'edit_button'));
        $form->addElement($submit);

        return $form;
    }
}