<?php
class QUESANS_CTRL_EditTopic extends OW_ActionController
{

    /**
     * Controller's default action
     *
     * @param array $params
     * @throws AuthorizationException
     * @throws Redirect404Exception
     */
    public function index( array $params = null )
    {
        $quesansService = QUESANS_BOL_QuesansService::getInstance();

        if ( !isset($params['id']) || !($topicId = (int) $params['id']) )
        {
            throw new Redirect404Exception();
        }

        $topicDto = $quesansService->findTopicById($topicId);

        if ( !$topicDto )
        {
            throw new Redirect404Exception();
        }

        $quesansGroup = $quesansService->getGroupInfo($topicDto->groupId);
        $quesansSection = $quesansService->findSectionById($quesansGroup->sectionId);

        $isHidden = $quesansSection->isHidden;

        $userId = OW::getUser()->getId();

        if ( $isHidden )
        {
            $isModerator = OW::getUser()->isAuthorized($quesansSection->entity);

            $eventParams = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'add_topic');
            $event = new OW_Event('quesans.check_permissions', $eventParams);
            OW::getEventManager()->trigger($event);
            $canPost = $event->getData();

            //check permissions
            $canEdit = OW::getUser()->isAuthorized($quesansSection->entity, 'add_topic') && $userId == $topicDto->userId;

            if ( !$isModerator )
            {
                if ( !$canPost )
                {
                    throw new AuthorizationException();
                }
                else if ( !$canEdit )
                {
                    $status = BOL_AuthorizationService::getInstance()->getActionStatus(
                        $quesansSection->entity, 'add_topic'
                    );
                    throw new AuthorizationException($status['msg']);
                }
            }
        }
        else
        {
            $isModerator = OW::getUser()->isAuthorized('quesans');
            $canEdit = OW::getUser()->isAuthorized('quesans', 'edit') && $userId == $topicDto->userId;

            if ( !$canEdit && !$isModerator )
            {
                throw new AuthorizationException();
            }
        }

        // first topic's post
        $postDto = $quesansService->findTopicFirstPost($topicId);
        $this->assign('post', $postDto);

        $uid = uniqid();
        $editTopicForm = $this->generateEditTopicForm($topicDto, $postDto, $uid);
        $this->addForm($editTopicForm);
        $lang = OW::getLanguage();
        $router = OW::getRouter();

        $topicInfo = $quesansService->getTopicInfo($topicId);
        $groupUrl = $router->urlForRoute('group-default', array('groupId' => $topicDto->groupId));
        $topicUrl = $router->urlForRoute('topic-default', array('topicId' => $topicDto->id));
        
        $lang->addKeyForJs('quesans', 'confirm_delete_attachment');

        $attachmentService = QUESANS_BOL_PostAttachmentService::getInstance();

        $enableAttachments = OW::getConfig()->getValue('quesans', 'enable_attachments');
        $this->assign('enableAttachments', $enableAttachments);

        if ( $enableAttachments )
        {
            $attachments = $attachmentService->findAttachmentsByPostIdList(array($postDto->id));
            $this->assign('attachments', $attachments);

            $attachmentCmp = new BASE_CLASS_FileAttachment('quesans', $uid);
            $this->addComponent('attachmentsCmp', $attachmentCmp);
        }
        
        if ( OW::getRequest()->isPost() && $editTopicForm->isValid($_POST) )
        {
            $values = $editTopicForm->getValues();
            
            $topicId = (int) $values['topic-id'];
            $postId = (int) $values['post-id'];
            $title = trim($values['title']);
            $text = trim($values['text']);

            $topicDto = $quesansService->findTopicById($topicId);
            $postDto = $quesansService->findPostById($postId);

            if ( $topicDto === null || $postDto === null || ($topicDto->userId != $userId && !$isModerator) )
            {
                exit();
            }

            //save topic
            $topicDto->title = strip_tags($title);
            $quesansService->saveOrUpdateTopic($topicDto);

            //save post
            $postDto->text = UTIL_HtmlTag::stripJs(UTIL_HtmlTag::stripTags($text, array('form', 'input', 'button'), null, true));
            $quesansService->saveOrUpdatePost($postDto);

            //save post edit info
            $editPostDto = $quesansService->findEditPost($postId);

            if ( $editPostDto === null )
            {
                $editPostDto = new QUESANS_BOL_EditPost();
            }

            $editPostDto->postId = $postId;
            $editPostDto->userId = $userId;
            $editPostDto->editStamp = time();
            $quesansService->saveOrUpdateEditPost($editPostDto);

            if ( $enableAttachments )
            {
                $filesArray = BOL_AttachmentService::getInstance()->getFilesByBundleName('quesans', $values['attachmentUid']);

                if ( $filesArray )
                {
                    $attachmentService = QUESANS_BOL_PostAttachmentService::getInstance();
                    $skipped = 0;

                    foreach ( $filesArray as $file )
                    {
                        $attachmentDto = new QUESANS_BOL_PostAttachment();
                        $attachmentDto->postId = $postDto->id;
                        $attachmentDto->fileName = $file['dto']->origFileName;
                        $attachmentDto->fileNameClean = $file['dto']->fileName;
                        $attachmentDto->fileSize = $file['dto']->size * 1024;
                        $attachmentDto->hash = uniqid();

                        $added = $attachmentService->addAttachment($attachmentDto, $file['path']);

                        if ( !$added )
                        {
                            $skipped++;
                        }
                    }

                    BOL_AttachmentService::getInstance()->deleteAttachmentByBundle('quesans', $values['attachmentUid']);
                    
                    if ( $skipped )
                    {
                        OW::getFeedback()->warning(OW::getLanguage()->text('quesans', 'not_all_attachments_added'));
                    }
                }
            }
            
            $params = array(
                'topicId' => $topicDto->id,
                'entity' => $quesansSection->entity ? $quesansSection->entity : NULL,
                'entityId' => $quesansGroup->entityId ? $quesansGroup->entityId : NULL,
                'userId' => $topicDto->userId,
                'topicUrl' => $topicUrl,
                'topicTitle' => $topicDto->title,
                'postText' => $postDto->text
            );
            $event = new OW_Event('quesans.topic_add', $params);
            OW::getEventManager()->trigger($event);

            OW::getEventManager()->trigger(new OW_Event('feed.action', array(
                'pluginKey' => 'quesans',
                'entityType' => 'quesans-topic',
                'entityId' => $topicDto->id,
                'userId' => $topicDto->userId,
                'time' => $postDto->createStamp
            )));

            OW::getEventManager()->trigger(new OW_Event(QUESANS_BOL_QuesansService::EVENT_AFTER_TOPIC_EDIT, array(
                'topicId' => $topicDto->id
            )));

            $this->redirect($topicUrl);
        }

        OW::getDocument()->setHeading(OW::getLanguage()->text('quesans', 'edit_topic_title'));
        OW::getDocument()->setHeadingIconClass('ow_ic_edit');

        $this->assign('isHidden', $isHidden);

        if ( $isHidden )
        {
            $event = new OW_Event('quesans.find_quesans_caption', array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId));
            OW::getEventManager()->trigger($event);

            $eventData = $event->getData();

            /** @var OW_Component $componentQuesansCaption */
            $componentQuesansCaption = $eventData['component'];

            if (!empty($componentQuesansCaption))
            {
                $this->assign('componentQuesansCaption', $componentQuesansCaption->render());
            }
            else
            {
                $componentQuesansCaption = false;
                $this->assign('componentQuesansCaption', $componentQuesansCaption);
            }

            $bcItems = array(
                array(
                    'href' => OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicId )),
                    'label' => OW::getLanguage()->text('quesans', 'back_to_topic')
                )
            );

            $breadCrumbCmp = new BASE_CMP_Breadcrumb($bcItems);
            $this->addComponent('breadcrumb', $breadCrumbCmp);

            OW::getNavigation()->deactivateMenuItems(OW_Navigation::MAIN);
            OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, $quesansSection->entity, $eventData['key']);
        }
        else
        {
            $bcItems = array(
                array(
                    'href' => $router->urlForRoute('quesans-default'),
                    'label' => $lang->text('quesans', 'quesans_index')
                ),
                array(
                    'href' => $router->urlForRoute('quesans-default') . '#section-' . $topicInfo['sectionId'],
                    'label' => $topicInfo['sectionName']
                ),
                array(
                    'href' => $groupUrl,
                    'label' => $topicInfo['groupName']
                ),
                array(
                    'href' => $topicUrl,
                    'label' => htmlspecialchars($topicDto->title)
                )
            );

            $breadCrumbCmp = new BASE_CMP_Breadcrumb($bcItems, $lang->text('quesans', 'topic_location'));
            $this->addComponent('breadcrumb', $breadCrumbCmp);

            OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
        }
    }

    /**
     * Generates edit topic form.
     *
     * @param $topicDto
     * @param $postDto
     * @param $uid
     * @return Form
     */
    private function generateEditTopicForm( $topicDto, $postDto, $uid )
    {
        $form = new Form('edit-topic-form');
        $form->setEnctype('multipart/form-data');
        
        $lang = OW::getLanguage();
        
        $topicIdField = new HiddenField('topic-id');
        $topicIdField->setValue($topicDto->id);
        $form->addElement($topicIdField);

        $postIdField = new HiddenField('post-id');
        $postIdField->setValue($postDto->id);
        $form->addElement($postIdField);

        $attachmentUid = new HiddenField('attachmentUid');
        $attachmentUid->setValue($uid);
        $attachmentUid->setRequired(true);
        $form->addElement($attachmentUid);

        $topicTitleField = new TextField('title');
        $topicTitleField->setValue($topicDto->title);
        $topicTitleField->setRequired(true);
        $sValidator = new StringValidator(1, 255);
        $sValidator->setErrorMessage($lang->text('quesans', 'chars_limit_exceeded', array('limit' => 255)));
        $topicTitleField->addValidator($sValidator);
        $form->addElement($topicTitleField);

        $btnSet = array(BOL_TextFormatService::WS_BTN_IMAGE, BOL_TextFormatService::WS_BTN_VIDEO, BOL_TextFormatService::WS_BTN_HTML);
        $postText = new WysiwygTextarea('text', $btnSet);
        $postText->setValue($postDto->text);
        $postText->setRequired(true);
        $sValidator = new StringValidator(1, 50000);
        $sValidator->setErrorMessage($lang->text('quesans', 'chars_limit_exceeded', array('limit' => 50000)));
        $postText->addValidator($sValidator);
        $form->addElement($postText);

        $submit = new Submit('save');
        $submit->setValue($lang->text('base', 'edit_button'));
        $form->addElement($submit);

        return $form;
    }
}
