<?php
class QUESANS_CTRL_Index extends OW_ActionController
{

    /**
     * Controller's default action
     */
    public function index()
    {
        $isModerator = OW::getUser()->isAuthorized('quesans');
        $viewPermissions = OW::getUser()->isAuthorized('quesans', 'view');

        if ( !$viewPermissions && !$isModerator )
        {
            $status = BOL_AuthorizationService::getInstance()->getActionStatus('quesans', 'view');
            throw new AuthorizationException($status['msg']);
        }

        $quesansService = QUESANS_BOL_QuesansService::getInstance();

        $this->assign('customizeUrl', OW::getRouter()->urlForRoute('customize-default'));
        $this->assign('isModerator', $isModerator);

        $userId = OW::getUser()->getId();
        $sectionGroupList = $quesansService->getSectionGroupList($userId);
                
        $singleQuesansMode = $quesansService->isSingleQuesansMode($sectionGroupList);
        $this->assign('singleMode', $singleQuesansMode);

        if ( $singleQuesansMode )
        {
            $firstSection = array_shift($sectionGroupList);
            $firstGroup = $firstSection['groups'][0];
            $groupId = $firstGroup['id'];

            $this->addComponent('groupCmp', new QUESANS_CMP_QuesansGroup(array('groupId' => $groupId, 'caption' => false)));

            $groupName = htmlspecialchars($firstGroup['name']);
            OW::getDocument()->setHeading(OW::getLanguage()->text('quesans', 'quesans_page_heading', array('quesans' => $groupName)));
            OW::getDocument()->setTitle($groupName);
            OW::getDocument()->setDescription(htmlspecialchars($firstGroup['description']));
        }
        else
        {
            $authors = array();
            foreach ( $sectionGroupList as $section )
            {
                foreach ( $section['groups'] as $group )
                {
                    if ( !$group['lastReply'] )
                    {
                        continue;
                    }
                    $id = $group['lastReply']['userId'];

                    if ( !in_array($id, $authors) )
                    {
                        array_push($authors, $id);
                    }
                }
            }

            $this->assign('sectionGroupList', $sectionGroupList);

            $userNames = BOL_UserService::getInstance()->getUserNamesForList($authors);
            $this->assign('userNames', $userNames);

            $displayNames = BOL_UserService::getInstance()->getDisplayNamesForList($authors);
            $this->assign('displayNames', $displayNames);

            OW::getDocument()->setHeading(OW::getLanguage()->text('quesans', 'quesans'));

            $this->addComponent('search', new QUESANS_CMP_QuesansSearch(array('scope' => 'all_quesans')));
        }

        $plugin = OW::getPluginManager()->getPlugin('quesans');
        $template = $plugin->getCtrlViewDir() . 'index.html';
        $this->setTemplate($template);

        OW::getDocument()->setHeadingIconClass('ow_ic_quesans');
        OW::getDocument()->setDescription(OW::getLanguage()->text('quesans', 'meta_description_quesanss'));
    }
}

