<?php
class QUESANS_CTRL_Search extends OW_ActionController
{
    private $quesansService;
    
    public function __construct()
    {
        parent::__construct();
        
        $this->quesansService = QUESANS_BOL_QuesansService::getInstance();
    }
    

    /**
     * Controller's default action
     * 
     * @param array $params
     */
    public function inQuesanss( array $params = null )
    {
        $plugin = OW::getPluginManager()->getPlugin('quesans');
        $this->setTemplate($plugin->getCtrlViewDir() . 'search_result.html');

        $lang = OW::getLanguage();
        
        $token = !empty($_GET['q']) && is_string($_GET['q']) ? urldecode(htmlspecialchars(trim($_GET['q']))) : null;
        $userToken = !empty($_GET['u']) && is_string($_GET['u']) ? urldecode(htmlspecialchars(trim($_GET['u']))) : null;

        if ( $token  || $userToken )
        {
            $this->assign('token', $token);
            $this->assign('userToken', $userToken);
            $tokenQuery = $token ? '&q=' . $token : null;
            $userTokenQuery = $userToken ? '&u=' . $userToken : null;
            $page = !empty($_GET['page']) && (int) $_GET['page'] ? abs((int) $_GET['page']) : 1;
            
            if ( OW::getUser()->isAuthorized('quesans') )
            {
                $excludeGroupIdList = array();
            }
            else 
            {
                $excludeGroupIdList = $this->quesansService->getPrivateUnavailableGroupIdList(OW::getUser()->getId());
            }
            
            $sortBy = !empty($_GET['sort']) && in_array($_GET['sort'], array('date', 'rel')) ? $_GET['sort'] : 'date';
            
            $topics = $this->quesansService->searchInGroups($token, $userToken, $page, $excludeGroupIdList, $sortBy);
            
            $authors = array();
            if ( $topics )
            {
                foreach ( $topics as &$topic )
                {
                    $topic['toolbar'] = array();
                    if ( !isset($topic['posts']) )
                    {
                        continue;
                    }
                    foreach ( $topic['posts'] as $post )
                    {
                        if ( !in_array($post['userId'], $authors) )
                        {
                            array_push($authors, $post['userId']);
                        }
                    }
                }
            }
            $this->assign('topics', $topics);
            
            // Paging
            $total = $this->quesansService->countFoundTopicsInGroups($token, $userToken, $excludeGroupIdList);
            $perPage = $this->quesansService->getTopicPerPageConfig();
            $pages = (int) ceil($total / $perPage);
            $paging = new BASE_CMP_Paging($page, $pages, $perPage);
            $this->assign('paging', $paging->render());
            
            // Sort control
            $sortCtrl = new BASE_CMP_SortControl();
            $url = OW::getRouter()->urlForRoute('quesans_search') . '?' . $tokenQuery . $userTokenQuery;
            $sortCtrl->addItem('date', $lang->text('quesans', 'sort_by_date'), $url.'&sort=date', !$sortBy || $sortBy == 'date');
            $sortCtrl->addItem('relevance', $lang->text('quesans', 'sort_by_relevance'), $url.'&sort=rel', $sortBy == 'rel');
            
            $this->addComponent('sort', $sortCtrl);
            
            $this->assign('avatars', BOL_AvatarService::getInstance()->getDataForUserAvatars($authors));
        }
        else 
        {
            $this->redirect(OW::getRouter()->urlForRoute('quesans-default'));
        }
        
        $this->addComponent('search', new QUESANS_CMP_QuesansSearch(
            array('scope' => 'all_quesans', 'token' => $token, 'userToken' => $userToken))
        );
        
        OW::getDocument()->setHeading($lang->text('quesans', 'search_page_heading'));
        OW::getDocument()->setHeadingIconClass('ow_ic_quesans');
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
    }
    
    public function inGroup( array $params = null )
    {
        $plugin = OW::getPluginManager()->getPlugin('quesans');
        $this->setTemplate($plugin->getCtrlViewDir() . 'search_result.html');

        $lang = OW::getLanguage();
        
        $token = !empty($_GET['q']) && is_string($_GET['q']) ? urldecode(htmlspecialchars(trim($_GET['q']))) : null;
        $userToken = !empty($_GET['u']) && is_string($_GET['u']) ? urldecode(htmlspecialchars(trim($_GET['u']))) : null;
        
        if ( $token || $userToken )
        {
            $this->assign('token', $token);
            $this->assign('userToken', $userToken);
            $tokenQuery = $token ? '&q=' . $token : null;
            $userTokenQuery = $userToken ? '&u=' . $userToken : null;
            $page = !empty($_GET['page']) && (int) $_GET['page'] ? abs((int) $_GET['page']) : 1;
            
            $groupId = (int)$params['groupId'];
            $quesansGroup = $this->quesansService->findGroupById($groupId);
            
            $userId = OW::getUser()->getId();

            $isHidden = $this->quesansService->groupIsHidden($groupId);
            
            if ( $isHidden )
            {
                $quesansSection = $this->quesansService->findSectionById($quesansGroup->sectionId);
                $isModerator = OW::getUser()->isAuthorized($quesansSection->entity);
                
                $event = new OW_Event('quesans.find_quesans_caption', array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId));
                OW::getEventManager()->trigger($event);
    
                $eventData = $event->getData();
                $componentQuesansCaption = $eventData['component'];
    
                $this->addComponent('componentQuesansCaption', $componentQuesansCaption);
            }
            else
            {
                $isModerator = OW::getUser()->isAuthorized('quesans');
            }
            
            if ( $quesansGroup->isPrivate )
            {
                if ( !$userId )
                {
                    throw new AuthorizationException();
                }
                else if ( !$isModerator )
                {
                    if ( !$this->quesansService->isPrivateGroupAvailable($userId, json_decode($quesansGroup->roles)) )
                    {
                        throw new AuthorizationException();
                    }
                }
            }
            
            $sortBy = !empty($_GET['sort']) && in_array($_GET['sort'], array('date', 'rel')) ? $_GET['sort'] : 'date'; 
            
            $topics = $this->quesansService->searchInGroup($token, $userToken, $page, $groupId, $isHidden, $sortBy);
            
            $authors = array();
            if ( $topics )
            {
                foreach ( $topics as &$topic )
                {
                    if ( !in_array($topic['userId'], $authors) )
                    {
                        array_push($authors, $topic['userId']);
                    }
                        
                    if ( !isset($topic['posts']) )
                    {
                        continue;
                    }
                    foreach ( $topic['posts'] as $post )
                    {
                        if ( !in_array($post['userId'], $authors) )
                        {
                            array_push($authors, $post['userId']);
                        }
                    }
                }
                
                $this->assign('avatars', BOL_AvatarService::getInstance()->getDataForUserAvatars($authors));
            }
            
            $this->assign('topics', $topics);
            
            // Paging
            $total = $this->quesansService->countFoundTopicsInGroup($token, $userToken, $groupId, $isHidden);
            $perPage = $this->quesansService->getTopicPerPageConfig();
            $pages = (int) ceil($total / $perPage);
            $paging = new BASE_CMP_Paging($page, $pages, $perPage);
            $this->assign('paging', $paging->render());
            
            // Sort control
            $sortCtrl = new BASE_CMP_SortControl();
            $url = OW::getRouter()->urlForRoute('quesans_search_group', array('groupId' => $groupId)) . '?' . $tokenQuery . $userTokenQuery;
            $sortCtrl->addItem('date', $lang->text('quesans', 'sort_by_date'), $url.'&sort=date', !$sortBy || $sortBy == 'date');
            $sortCtrl->addItem('relevance', $lang->text('quesans', 'sort_by_relevance'), $url.'&sort=rel', $sortBy == 'rel');
            
            $this->addComponent('sort', $sortCtrl);
        }
        else
        {
            $this->redirect(OW::getRouter()->urlForRoute('quesans-default'));
        }
        
        $this->addComponent('search', new QUESANS_CMP_QuesansSearch(
            array('scope' => 'group', 'token' => $token, 'userToken' => $userToken, 'groupId' => $groupId))
        );
        
        OW::getDocument()->setHeading($lang->text('quesans', 'search_page_heading'));
        OW::getDocument()->setHeadingIconClass('ow_ic_quesans');
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
    }

    public function inSection( array $params = null )
    {
        $plugin = OW::getPluginManager()->getPlugin('quesans');
        $this->setTemplate($plugin->getCtrlViewDir() . 'search_result.html');

        $lang = OW::getLanguage();

        $token = !empty($_GET['q']) && is_string($_GET['q']) ? urldecode(htmlspecialchars(trim($_GET['q']))) : null;
        $userToken = !empty($_GET['u']) && is_string($_GET['u']) ? urldecode(htmlspecialchars(trim($_GET['u']))) : null;

        if ( $token || $userToken )
        {
            $this->assign('token', $token);
            $this->assign('userToken', $userToken);
            $tokenQuery = $token ? '&q=' . $token : null;
            $userTokenQuery = $userToken ? '&u=' . $userToken : null;

            $sectionId = (int) $params['sectionId'];
            $page = !empty($_GET['page']) && (int) $_GET['page'] ? abs((int) $_GET['page']) : 1;

            if ( OW::getUser()->isAuthorized('quesans') )
            {
                $excludeGroupIdList = array();
            }
            else
            {
                $excludeGroupIdList = $this->quesansService->getPrivateUnavailableGroupIdList(OW::getUser()->getId());
            }

            $sortBy = !empty($_GET['sort']) && in_array($_GET['sort'], array('date', 'rel')) ? $_GET['sort'] : 'date';

            $topics = $this->quesansService->searchInSection($token, $userToken, $sectionId, $page, $excludeGroupIdList, $sortBy);

            $authors = array();
            if ( $topics )
            {
                foreach ( $topics as &$topic )
                {
                    $topic['toolbar'] = array();
                    if ( !isset($topic['posts']) )
                    {
                        continue;
                    }
                    foreach ( $topic['posts'] as $post )
                    {
                        if ( !in_array($post['userId'], $authors) )
                        {
                            array_push($authors, $post['userId']);
                        }
                    }
                }
            }
            $this->assign('topics', $topics);

            // Paging
            $total = $this->quesansService->countFoundTopicsInSection($token, $userToken, $sectionId, $excludeGroupIdList);
            $perPage = $this->quesansService->getTopicPerPageConfig();
            $pages = (int) ceil($total / $perPage);
            $paging = new BASE_CMP_Paging($page, $pages, $perPage);
            $this->assign('paging', $paging->render());

            // Sort control
            $sortCtrl = new BASE_CMP_SortControl();
            $url = OW::getRouter()->urlForRoute('quesans_search_section', array('sectionId' => $params['sectionId'])) . '?' . $tokenQuery . $userTokenQuery;
            $sortCtrl->addItem('date', $lang->text('quesans', 'sort_by_date'), $url.'&sort=date', !$sortBy || $sortBy == 'date');
            $sortCtrl->addItem('relevance', $lang->text('quesans', 'sort_by_relevance'), $url.'&sort=rel', $sortBy == 'rel');

            $this->addComponent('sort', $sortCtrl);

            $this->assign('avatars', BOL_AvatarService::getInstance()->getDataForUserAvatars($authors));
        }
        else
        {
            $this->redirect(OW::getRouter()->urlForRoute('quesans-default'));
        }

        $this->addComponent('search', new QUESANS_CMP_QuesansSearch(
            array('scope' => 'section', 'sectionId' => $sectionId, 'token' => $token, 'userToken' => $userToken))
        );
        
        OW::getDocument()->setHeading($lang->text('quesans', 'search_page_heading'));
        OW::getDocument()->setHeadingIconClass('ow_ic_quesans');
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
    }
    
    public function inTopic( array $params = null )
    {
        $plugin = OW::getPluginManager()->getPlugin('quesans');
        $this->setTemplate($plugin->getCtrlViewDir() . 'search_result.html');

        $lang = OW::getLanguage();
        
        $token = !empty($_GET['q']) && is_string($_GET['q']) ? urldecode(htmlspecialchars(trim($_GET['q']))) : null;
        $userToken = !empty($_GET['u']) && is_string($_GET['u']) ? urldecode(htmlspecialchars(trim($_GET['u']))) : null;
        
        if ( $token || $userToken )
        {
            $this->assign('token', $token);
            $this->assign('userToken', $userToken);
            $tokenQuery = $token ? '&q=' . $token : null;
            $userTokenQuery = $userToken ? '&u=' . $userToken : null;
            
            $topicId = (int)$params['topicId'];
            $userId = OW::getUser()->getId();
            
            $topic = $this->quesansService->findTopicById($topicId);
            $quesansGroup = $this->quesansService->findGroupById($topic->groupId);
            $quesansSection = $this->quesansService->findSectionById($quesansGroup->sectionId);
            
            if ( $quesansSection && $quesansSection->isHidden )
            {
                $event = new OW_Event('quesans.find_quesans_caption', array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId));
                OW::getEventManager()->trigger($event);
    
                $eventData = $event->getData();
                $componentQuesansCaption = $eventData['component'];
    
                $this->addComponent('componentQuesansCaption', $componentQuesansCaption);
                
                $isModerator = OW::getUser()->isAuthorized($quesansSection->entity);
            }
            else 
            {
                $isModerator = OW::getUser()->isAuthorized('quesans');
            }
            
            if ( $quesansGroup->isPrivate )
            {
                if ( !$userId )
                {
                    throw new AuthorizationException();
                } 
                else if ( !$isModerator )
                {
                    if ( !$this->quesansService->isPrivateGroupAvailable($userId, json_decode($quesansGroup->roles)) )
                    {
                        throw new AuthorizationException();
                    }
                }
            }
            
            $sortBy = !empty($_GET['sort']) && in_array($_GET['sort'], array('date', 'rel')) ? $_GET['sort'] : 'date';
            
            $topics = $this->quesansService->searchInTopic($token, $userToken, $topicId, $sortBy);
            
            if ( $topics )
            {
                $authors = array();
                foreach ( $topics as &$topic )
                {
                    if ( !in_array($topic['userId'], $authors) )
                    {
                        array_push($authors, $topic['userId']);
                    }
                        
                    if ( !isset($topic['posts']) )
                    {
                        continue;
                    }
                    foreach ( $topic['posts'] as $post )
                    {
                        if ( !in_array($post['userId'], $authors) )
                        {
                            array_push($authors, $post['userId']);
                        }
                    }
                }
                
                $this->assign('avatars', BOL_AvatarService::getInstance()->getDataForUserAvatars($authors));
            }
            
            $this->assign('topics', $topics);
            
            // Sort control
            $sortCtrl = new BASE_CMP_SortControl();
            $url = OW::getRouter()->urlForRoute('quesans_search_topic', array('topicId' => $topicId)) . '?' . $tokenQuery . $userTokenQuery;
            $sortCtrl->addItem('date', $lang->text('quesans', 'sort_by_date'), $url.'&sort=date', !$sortBy || $sortBy == 'date');
            $sortCtrl->addItem('relevance', $lang->text('quesans', 'sort_by_relevance'), $url.'&sort=rel', $sortBy == 'rel');
            
            $this->addComponent('sort', $sortCtrl);
        }
        else 
        {
            $this->redirect(OW::getRouter()->urlForRoute('quesans-default'));
        }
        
        $this->addComponent('search', new QUESANS_CMP_QuesansSearch(
            array('scope' => 'topic', 'token' => $token, 'userToken' => $userToken, 'topicId' => $topicId))
        );
        
        OW::getDocument()->setHeading($lang->text('quesans', 'search_page_heading'));
        OW::getDocument()->setHeadingIconClass('ow_ic_quesans');
        OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
    }
}
