<?php
class QUESANS_CTRL_Section extends OW_ActionController
{
    /**
     * @var QUESANS_BOL_QuesansService
     */
    private $quesansService;

    /**
     * Class constructor
     */
    public function __construct()
    {
        parent::__construct();

        $this->quesansService = QUESANS_BOL_QuesansService::getInstance();

        if ( !OW::getRequest()->isAjax() )
        {
            OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
        }
    }

    /**
     * Controller's default action
     *
     * @param array $params
     * @throws Redirect404Exception
     */
    public function index( array $params )
    {
        if ( !isset($params['sectionId']) || !($sectionId = (int) $params['sectionId']) )
        {
            throw new Redirect404Exception();
        }
        
        $quesansSection = $this->quesansService->findSectionById($sectionId);
        if ( !$quesansSection || $quesansSection->isHidden )
        {
            throw new Redirect404Exception();
        }

        $userId = OW::getUser()->getId();

        $bcItems = array(
            array(
                'href' => OW::getRouter()->urlForRoute('quesans-default'),
                'label' => OW::getLanguage()->text('quesans', 'quesans_index')
            ),
            array(
                'label' => $quesansSection->name
            )
        );

        $breadCrumbCmp = new BASE_CMP_Breadcrumb($bcItems);
        $this->addComponent('breadcrumb', $breadCrumbCmp);

        $sectionGroupList = $this->quesansService->getSectionGroupList($userId, $sectionId);

        $authors = array();
        foreach ( $sectionGroupList as $section )
        {
            foreach ( $section['groups'] as $group )
            {
                if ( !$group['lastReply'] )
                {
                    continue;
                }
                $id = $group['lastReply']['userId'];

                if ( !in_array($id, $authors) )
                {
                    array_push($authors, $id);
                }
            }
        }

        $this->assign('sectionGroupList', $sectionGroupList);

        $userNames = BOL_UserService::getInstance()->getUserNamesForList($authors);
        $this->assign('userNames', $userNames);

        $displayNames = BOL_UserService::getInstance()->getDisplayNamesForList($authors);
        $this->assign('displayNames', $displayNames);

        $this->addComponent('search', new QUESANS_CMP_QuesansSearch(array('scope' => 'section', 'sectionId' => $sectionId)));

        OW::getDocument()->setHeading(OW::getLanguage()->text('quesans', 'quesans'));
        OW::getDocument()->setHeadingIconClass('ow_ic_quesans');
    }
}
