<?php
class QUESANS_CTRL_Topic extends OW_ActionController
{
    private $quesansService;

    /**
     * Class constructor
     */
    public function __construct()
    {
        parent::__construct();

        $this->quesansService = QUESANS_BOL_QuesansService::getInstance();

        if ( !OW::getRequest()->isAjax() )
        {
            OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, 'quesans', 'quesans');
        }
    }

    /**
     * Controller's default action
     *
     * @param array $params
     * @throws AuthorizationException
     * @throws Redirect404Exception
     */
    public function index( array $params )
    {
        if ( !isset($params['topicId']) || ($topicDto = $this->quesansService->findTopicById($params['topicId'])) === null )
        {
            throw new Redirect404Exception();
        }

        if ( $topicDto != QUESANS_BOL_QuesansService::STATUS_APPROVED )
        {
            //throw new Redirect404Exception();
        }

        $quesansGroup = $this->quesansService->findGroupById($topicDto->groupId);
        $quesansSection = $this->quesansService->findSectionById($quesansGroup->sectionId);

        $isHidden = $quesansSection->isHidden;

        $userId = OW::getUser()->getId();
        $isOwner = ( $topicDto->userId == $userId ) ? true : false;

        $postReplyPermissionErrorText = null;

        if ( $isHidden )
        {
            $event = new OW_Event('quesans.can_view', array(
                'entity' => $quesansSection->entity,
                'entityId' => $quesansGroup->entityId
                ), true);
            OW::getEventManager()->trigger($event);

            $canView = $event->getData();

            $isModerator = OW::getUser()->isAuthorized($quesansSection->entity);

            $params = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'edit_topic');
            $event = new OW_Event('quesans.check_permissions', $params);
            OW::getEventManager()->trigger($event);
            $canEdit = $event->getData();

            $params = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'add_topic');
            $event = new OW_Event('quesans.check_permissions', $params);
            OW::getEventManager()->trigger($event);

            $canPost = $event->getData();

            $postReplyPermissionErrorText = OW::getLanguage()->text($quesansSection->entity, 'post_reply_permission_error');

            $canMoveToHidden = BOL_AuthorizationService::getInstance()->isActionAuthorized($quesansSection->entity, 'move_topic_to_hidden') && $isModerator;

            //$eventParams = array('pluginKey' => $quesansSection->entity, 'action' => 'add_post');
            //TODO Zaph:create action that will check if user allowed to delete post separately from topic
        }
        else
        {
            $isModerator = OW::getUser()->isAuthorized('quesans');

            $canView = OW::getUser()->isAuthorized('quesans', 'view');
            $canEdit = $isOwner || $isModerator;
            $canPost = OW::getUser()->isAuthorized('quesans', 'edit');
            $canMoveToHidden = BOL_AuthorizationService::getInstance()->isActionAuthorized('quesans', 'move_topic_to_hidden') && $isModerator;

            //$eventParams = array('pluginKey' => 'quesans', 'action' => 'add_post');
        }

        $canLock = $canSticky = $isModerator;

        if ( !$canView && !$isModerator )
        {
            $status = BOL_AuthorizationService::getInstance()->getActionStatus('quesans', 'view');
            throw new AuthorizationException($status['msg']);
        }

        if ( $quesansGroup->isPrivate )
        {
            if ( !$userId )
            {
                throw new AuthorizationException();
            }
            else if ( !$isModerator )
            {
                if ( !$this->quesansService->isPrivateGroupAvailable($userId, json_decode($quesansGroup->roles)) )
                {
                    throw new AuthorizationException();
                }
            }
        }

        $page = !empty($_GET['page']) && (int) $_GET['page'] ? abs((int) $_GET['page']) : 1;

        //update topic's view count
        $topicDto->viewCount += 1;
        $this->quesansService->saveOrUpdateTopic($topicDto);

        //update user read info
        $this->quesansService->setTopicRead($topicDto->id, $userId);

        $topicInfo = $this->quesansService->getTopicInfo($topicDto->id);
        $postList = $this->quesansService->getTopicPostList($topicDto->id, $page);

        OW::getEventManager()->trigger(new OW_Event('quesans.topic_post_list', array('list' => $postList)));

        $this->assign('isHidden', $isHidden);

        // adds quesans caption if any
        if ( $isHidden )
        {
            $event = new OW_Event('quesans.find_quesans_caption', array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId));
            OW::getEventManager()->trigger($event);

            $eventData = $event->getData();

            /** @var OW_Component $componentQuesansCaption */
            $componentQuesansCaption = $eventData['component'];

            if ( !empty($componentQuesansCaption) )
            {
                $this->assign('componentQuesansCaption', $componentQuesansCaption->render());
            }
            else
            {
                $componentQuesansCaption = false;
                $this->assign('componentQuesansCaption', $componentQuesansCaption);
            }

            $eParams = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'edit_topic');
            $event = new OW_Event('quesans.check_permissions', $eParams);
            OW::getEventManager()->trigger($event);
            if ( $event->getData() )
            {
                $canLock = $canSticky = true;
            }
        }

        $this->assign('postReplyPermissionErrorText', $postReplyPermissionErrorText);
        $this->assign('isHidden', $isHidden);
        $this->assign('isOwner', $isOwner);
        $this->assign('canPost', $canPost);
        $this->assign('canLock', $canLock);
        $this->assign('canSticky', $canSticky);
        $this->assign('canSubscribe', OW::getUser()->isAuthorized('quesans', 'subscribe'));
        $this->assign('isSubscribed', $userId && QUESANS_BOL_SubscriptionService::getInstance()->isUserSubscribed($userId, $topicDto->id));

        if ( !$postList )
        {
            throw new Redirect404Exception();
        }

        $toolbars = array();
        $lang = OW::getLanguage();

        $langQuote = $lang->text('quesans', 'quote');
        $langFlag = $lang->text('base', 'flag');
        $langEdit = $lang->text('quesans', 'edit');
        $langDelete = $lang->text('quesans', 'delete');

        $iteration = 0;
        $userIds = array();
        $postIds = array();
        $flagItems = array();

        $firstTopicPost = $this->quesansService->findTopicFirstPost($topicDto->id);

        foreach ( $postList as &$post )
        {
            $post['text'] = UTIL_HtmlTag::autoLink($post['text']);
            $post['permalink'] = $this->quesansService->getPostUrl($post['topicId'], $post['id'], true, $page);
            $post['number'] = ($page - 1) * $this->quesansService->getPostPerPageConfig() + $iteration + 1;

            if ( $iteration == 0 )
            {
                $firstPostText = substr(htmlspecialchars(strip_tags($post['text'])), 0, 154);
            }

            // get list of users
            if ( !in_array($post['userId'], $userIds) )
                $userIds[$post['userId']] = $post['userId'];

            $toolbar = array();

            array_push($toolbar, array('class' => 'post_permalink', 'href' => $post['permalink'], 'label' => '#' . $post['number']));

            if ( $userId )
            {
                if ( !$topicDto->locked && ($canEdit || $canPost) )
                {
                    array_push($toolbar, array('id' => $post['id'], 'class' => 'quote_post', 'href' => 'javascript://', 'label' => $langQuote));
                }

                if ( $userId != (int) $post['userId'] )
                {
                    $lagItemKey = 'flag_' . $post['id'];
                    $flagItems[$lagItemKey] = array(
                        'id' => $post['id'],
                        'title' => $post['text'],
                        'href' => $this->quesansService->getPostUrl($post['topicId'], $post['id'])
                    );

                    array_push($toolbar, array('label' => $langFlag, 'href' => 'javascript://', 'id' => $lagItemKey, 'class' => 'post_flag_item'));
                }
            }

            if ( $isModerator || ($userId == (int) $post['userId'] && !$topicDto->locked) )
            {
                $href = $iteration == 0 && $page == 1 ?
                    OW::getRouter()->urlForRoute('edit-topic', array('id' => $post['topicId'])) :
                    OW::getRouter()->urlForRoute('edit-post', array('id' => $post['id']));

                array_push($toolbar, array('id' => $post['id'], 'href' => $href, 'label' => $langEdit));

                if ( !($iteration == 0 && $page == 1) )
                {
                    array_push($toolbar, array('id' => $post['id'], 'class' => 'delete_post', 'href' => 'javascript://', 'label' => $langDelete));
                }

                if ( $iteration === 0 && !$isOwner && $isModerator && $topicInfo['status'] == QUESANS_BOL_QuesansService::STATUS_APPROVAL )
                {
                    $toolbar[] = array('id' => $topicInfo['id'], 'href' => OW::getRouter()->urlForRoute('quesans_approve_topic', array('id' => $topicInfo['id'])), 'label' => OW::getLanguage()->text('quesans', 'approve_topic'));
                }
            }

            $toolbars[$post['id']] = $toolbar;

            if ( count($post['edited']) && !in_array($post['edited']['userId'], $userIds) )
                $userIds[$post['edited']['userId']] = $post['edited']['userId'];

            $iteration++;

            array_push($postIds, $post['id']);
        }

        OW::getDocument()->addScript(OW::getPluginManager()->getPlugin('base')->getStaticJsUrl() . 'jquery-fieldselection.js');

        $js = UTIL_JsGenerator::newInstance()
            ->newVariable('flagItems', $flagItems)
            ->jQueryEvent(
            '.post_flag_item a', 'click', 'var inf = flagItems[this.id];
                if (inf.id == '.$firstTopicPost->id.' ){
                    OW.flagContent("'.QUESANS_BOL_QuesansService::FEED_ENTITY_TYPE.'", '.$firstTopicPost->topicId.');
                }
                else{
                    OW.flagContent("'.QUESANS_BOL_QuesansService::FEED_POST_ENTITY_TYPE.'", inf.id);
                }'
        );

        OW::getDocument()->addOnloadScript($js, 1001);

        $this->assign('toolbars', $toolbars);

        $avatars = BOL_AvatarService::getInstance()->getDataForUserAvatars($userIds);
        $this->assign('avatars', $avatars);

        $enableAttachments = OW::getConfig()->getValue('quesans', 'enable_attachments');
        $this->assign('enableAttachments', $enableAttachments);

        $uid = uniqid();
        $addPostForm = $this->generateAddPostForm($topicDto->id, $uid);
        $this->addForm($addPostForm);

        $addPostInputId = $addPostForm->getElement('text')->getId();

        if ( $enableAttachments )
        {
            $attachments = QUESANS_BOL_PostAttachmentService::getInstance()->findAttachmentsByPostIdList($postIds);
            $this->assign('attachments', $attachments);

            $attachmentCmp = new BASE_CLASS_FileAttachment('quesans', $uid);
            $this->addComponent('attachmentsCmp', $attachmentCmp);
        }

        $plugin = OW::getPluginManager()->getPlugin('quesans');

        $indexUrl = OW::getRouter()->urlForRoute('quesans-default');
        $groupUrl = OW::getRouter()->urlForRoute('group-default', array('groupId' => $topicDto->groupId));
        $deletePostUrl = OW::getRouter()->urlForRoute('delete-post', array('topicId' => $topicDto->id, 'postId' => 'postId'));
        $stickyTopicUrl = OW::getRouter()->urlForRoute('sticky-topic', array('topicId' => $topicDto->id, 'page' => $page));
        $lockTopicUrl = OW::getRouter()->urlForRoute('lock-topic', array('topicId' => $topicDto->id, 'page' => $page));
        $deleteTopicUrl = OW::getRouter()->urlForRoute('delete-topic', array('topicId' => $topicDto->id));
        $getPostUrl = OW::getRouter()->urlForRoute('get-post', array('postId' => 'postId'));
        $moveTopicUrl = OW::getRouter()->urlForRoute('move-topic');
        $subscribeTopicUrl = OW::getRouter()->urlForRoute('subscribe-topic', array('id' => $topicDto->id));
        $unsubscribeTopicUrl = OW::getRouter()->urlForRoute('unsubscribe-topic', array('id' => $topicDto->id));

        $topicInfoJs = json_encode(array('sticky' => $topicDto->sticky, 'locked' => $topicDto->locked, 'ishidden' => $isHidden && !$canMoveToHidden));

        $onloadJs = "
			QuesansTopic.deletePostUrl = '$deletePostUrl';
			QuesansTopic.stickyTopicUrl = '$stickyTopicUrl';
			QuesansTopic.lockTopicUrl = '$lockTopicUrl';
			QuesansTopic.subscribeTopicUrl = '$subscribeTopicUrl';
			QuesansTopic.unsubscribeTopicUrl = '$unsubscribeTopicUrl';
			QuesansTopic.deleteTopicUrl = '$deleteTopicUrl';
			QuesansTopic.getPostUrl = '$getPostUrl';
			QuesansTopic.add_post_input_id = '$addPostInputId';
			QuesansTopic.construct($topicInfoJs);
			";

        OW::getDocument()->addOnloadScript($onloadJs);

        OW::getDocument()->addScript($plugin->getStaticJsUrl() . "quesans.js");

        // add language keys for javascript
        $lang->addKeyForJs('quesans', 'sticky_topic_confirm');
        $lang->addKeyForJs('quesans', 'unsticky_topic_confirm');
        $lang->addKeyForJs('quesans', 'lock_topic_confirm');
        $lang->addKeyForJs('quesans', 'unlock_topic_confirm');
        $lang->addKeyForJs('quesans', 'delete_topic_confirm');
        $lang->addKeyForJs('quesans', 'delete_post_confirm');
        $lang->addKeyForJs('quesans', 'edit_topic_title');
        $lang->addKeyForJs('quesans', 'edit_post_title');
        $lang->addKeyForJs('quesans', 'move_topic_title');
        $lang->addKeyForJs('quesans', 'confirm_delete_attachment');
        $lang->addKeyForJs('quesans', 'quesans_quote');
        $lang->addKeyForJs('quesans', 'quesans_quote_from');

        // first topic's post
        $postDto = $firstTopicPost;

        //posts count on page
        $count = $this->quesansService->getPostPerPageConfig();

        $postCount = $this->quesansService->findTopicPostCount($topicDto->id);
        $pageCount = ceil($postCount / $count);

        $groupSelect = $this->quesansService->getGroupSelectList($topicDto->groupId, $canMoveToHidden, $userId);
        $moveTopicForm = $this->generateMoveTopicForm($moveTopicUrl, $groupSelect, $topicDto);
        $this->addForm($moveTopicForm);

        $Paging = new BASE_CMP_Paging($page, $pageCount, $count);

        $this->assign('paging', $Paging->render());

        if ( $isHidden )
        {
            OW::getNavigation()->deactivateMenuItems(OW_Navigation::MAIN);
            OW::getNavigation()->activateMenuItem(OW_Navigation::MAIN, $quesansSection->entity, $eventData['key']);

            OW::getDocument()->setHeading(OW::getLanguage()->text($quesansSection->entity, 'topic_page_heading', array(
                'topic' => $topicInfo['title'],
                'group' => $topicInfo['groupName'],
                'content' => ''
            )));

            $bcItems = array(
                array(
                    'href' => OW::getRouter()->urlForRoute('group-default', array('groupId' => $quesansGroup->getId())),
                    'label' => OW::getLanguage()->text($quesansSection->entity, 'view_all_topics')
                )
            );

            $breadCrumbCmp = new BASE_CMP_Breadcrumb($bcItems);
            $this->addComponent('breadcrumb', $breadCrumbCmp);
        }
        else
        {
            $bcItems = array(
                array(
                    'href' => $indexUrl,
                    'label' => $lang->text('quesans', 'quesans_index')
                ),
                array(
                    'href' => OW::getRouter()->urlForRoute('section-default', array('sectionId' => $topicInfo['sectionId'])),
                    'label' => $topicInfo['sectionName']
                ),
                array(
                    'href' => $groupUrl,
                    'label' => $topicInfo['groupName']
                )
            );

            $breadCrumbCmp = new BASE_CMP_Breadcrumb($bcItems, $lang->text('quesans', 'topic_location'));
            $this->addComponent('breadcrumb', $breadCrumbCmp);

            OW::getDocument()->setHeading(OW::getLanguage()->text('quesans', 'topic_page_heading', array(
                'topic' => $topicInfo['title'],
                'content' => $topicInfo['status'] == QUESANS_BOL_QuesansService::STATUS_APPROVED ? '' : OW::getLanguage()->text('quesans', 'pending_approval')
            )));
        }

        OW::getDocument()->setHeadingIconClass('ow_ic_script');

        $this->assign('indexUrl', $indexUrl);
        $this->assign('groupUrl', $groupUrl);

        $this->assign('topicInfo', $topicInfo);
        $this->assign('postList', $postList);
        $this->assign('page', $page);

        $this->assign('userId', $userId);
        $this->assign('isModerator', $isModerator);
        $this->assign('canEdit', $canEdit);
        $this->assign('canMoveToHidden', $canMoveToHidden);

        OW::getDocument()->setTitle($topicInfo['title']);
        OW::getDocument()->setDescription($firstPostText);

        $this->addComponent('search', new QUESANS_CMP_QuesansSearch(array('scope' => 'topic', 'topicId' => $topicDto->id)));

        $tb = array();

        $toolbarEvent = new BASE_CLASS_EventCollector('quesans.collect_topic_toolbar_items', array(
            'topicId' => $topicDto->id,
            'topicDto' => $topicDto
        ));

        OW::getEventManager()->trigger($toolbarEvent);

        foreach ( $toolbarEvent->getData() as $toolbarItem )
        {
            array_push($tb, $toolbarItem);
        }
        $this->assign('tb', $tb);
    }

    public function ajaxDeleteAttachment()
    {
        $result = array('result' => false);

        if ( !isset($_POST['attachmentId']) )
        {
            exit(json_encode($result));
        }

        $attachmentService = QUESANS_BOL_PostAttachmentService::getInstance();
        $quesansService = QUESANS_BOL_QuesansService::getInstance();
        $lang = OW::getLanguage();

        $attachment = $attachmentService->findPostAttachmentById((int) $_POST['attachmentId']);

        if ( $attachment )
        {
            $userId = OW::getUser()->getId();
            $isModerator = OW::getUser()->isAuthorized('quesans');

            $post = $quesansService->findPostById($attachment->postId);

            if ( $post )
            {
                if ( $isModerator || $post->userId == $userId )
                {
                    $attachmentService->deleteAttachment($attachment->id);

                    $result = array('result' => true, 'msg' => $lang->text('quesans', 'attachment_deleted'));
                }
            }
        }
        else
        {
            $result = array('result' => false);
        }

        exit(json_encode($result));
    }

    /**
     * This action adds a post and after execution redirects to default action
     *
     * @param array $params
     * @throws Redirect404Exception
     * @throws AuthenticateException
     */
    public function addPost( array $params )
    {
        if ( !isset($params['topicId']) || !($topicId = (int) $params['topicId']) )
        {
            throw new Redirect404Exception();
        }

        $topicDto = $this->quesansService->findTopicById($topicId);

        if ( !$topicDto )
        {
            throw new Redirect404Exception();
        }

        $uid = $params['uid'];

        $addPostForm = $this->generateAddPostForm($topicId, $uid);

        if ( OW::getRequest()->isPost() && $addPostForm->isValid($_POST) )
        {
            $data = $addPostForm->getValues();

            if ( $data['topic'] && $data['topic'] == $topicDto->id && !$topicDto->locked )
            {
                if ( !OW::getUser()->getId() )
                {
                    throw new AuthenticateException();
                }

                $postDto = new QUESANS_BOL_Post();
                $postDto->topicId = $data['topic'];
                $postDto->userId = OW::getUser()->getId();
                $postDto->text = UTIL_HtmlTag::stripJs(UTIL_HtmlTag::stripTags($data['text'], array('form', 'input', 'button'), null, true));

                $postDto->createStamp = time();
                $this->quesansService->saveOrUpdatePost($postDto);

                $topicDto->lastPostId = $postDto->getId();
                $this->quesansService->saveOrUpdateTopic($topicDto);

                $this->quesansService->deleteByTopicId($topicId);

                $enableAttachments = OW::getConfig()->getValue('quesans', 'enable_attachments');

                if ( $enableAttachments )
                {
                    $filesArray = BOL_AttachmentService::getInstance()->getFilesByBundleName('quesans', $data['attachmentUid']);

                    if ( $filesArray )
                    {
                        $attachmentService = QUESANS_BOL_PostAttachmentService::getInstance();
                        $skipped = 0;

                        foreach ( $filesArray as $file )
                        {
                            $attachmentDto = new QUESANS_BOL_PostAttachment();
                            $attachmentDto->postId = $postDto->id;
                            $attachmentDto->fileName = $file['dto']->origFileName;
                            $attachmentDto->fileNameClean = $file['dto']->fileName;
                            $attachmentDto->fileSize = $file['dto']->size * 1024;
                            $attachmentDto->hash = uniqid();

                            $added = $attachmentService->addAttachment($attachmentDto, $file['path']);

                            if ( !$added )
                            {
                                $skipped++;
                            }
                        }

                        BOL_AttachmentService::getInstance()->deleteAttachmentByBundle('quesans', $data['attachmentUid']);

                        if ( $skipped )
                        {
                            OW::getFeedback()->warning(OW::getLanguage()->text('quesans', 'not_all_attachments_added'));
                        }
                    }
                }

                $postUrl = $this->quesansService->getPostUrl($topicId, $postDto->id);

                $event = new OW_Event('quesans.add_post', array('postId' => $postDto->id, 'topicId' => $topicId, 'userId' => $postDto->userId));
                OW::getEventManager()->trigger($event);

                $quesansGroup = $this->quesansService->findGroupById($topicDto->groupId);
                if ( $quesansGroup )
                {
                    $quesansSection = $this->quesansService->findSectionById($quesansGroup->sectionId);
                    if ( $quesansSection )
                    {
                        $pluginKey = $quesansSection->isHidden ? $quesansSection->entity : 'quesans';
                        $action = $quesansSection->isHidden ? 'add_topic' : 'edit';
                        BOL_AuthorizationService::getInstance()->trackAction($pluginKey, $action);
                    }
                }

                $this->redirect($postUrl);
            }
        }
        else
        {
            $this->redirect(OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicId)));
        }
    }

    /**
     * This action deletes thread post
     * and after execution redirects to default action
     *
     * @param array $params
     * @throws Redirect404Exception
     */
    public function deletePost( array $params )
    {
        if ( !isset($params['topicId']) || !($topicId = (int) $params['topicId']) || !isset($params['postId']) || !($postId = (int) $params['postId']) )
        {
            throw new Redirect404Exception();
        }

        $topicDto = $this->quesansService->findTopicById($topicId);
        $postDto = $this->quesansService->findPostById($postId);

        $userId = OW::getUser()->getId();
        $isModerator = OW::getUser()->isAuthorized('quesans');

        $quesansGroup = $this->quesansService->findGroupById($topicDto->groupId);
        $quesansSection = $this->quesansService->findSectionById($quesansGroup->sectionId);

        if ( $quesansSection->isHidden )
        {
            $eParams = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'edit_topic');
            $event = new OW_Event('quesans.check_permissions', $eParams);
            OW::getEventManager()->trigger($event);

            if ( $event->getData() )
            {
                $isModerator = true;
            }
        }

        if ( $topicDto && $postDto && ($postDto->userId == $userId || $isModerator) )
        {
            $prevPostDto = $this->quesansService->findPreviousPost($topicId, $postId);

            $topicDto->lastPostId = $prevPostDto->id;
            $this->quesansService->saveOrUpdateTopic($topicDto);

            $this->quesansService->deletePost($postId);
            $postUrl = $this->quesansService->getPostUrl($topicId, $prevPostDto->id, false);
        }
        else
        {
            $postUrl = $this->quesansService->getPostUrl($topicId, $postId, false);
        }

        $this->redirect($postUrl);
    }

    /**
     * This action sets the topic sticky or unsticky
     * and after execution redirects to default action
     *
     * @param array $params
     * @throws Redirect404Exception
     */
    public function stickyTopic( array $params )
    {
        if ( !isset($params['topicId']) || !($topicId = (int) $params['topicId']) || !isset($params['page']) || !($page = (int) $params['page']) )
        {
            throw new Redirect404Exception();
        }
        $isModerator = OW::getUser()->isAuthorized('quesans');

        $topicDto = $this->quesansService->findTopicById($topicId);

        if ( $topicDto )
        {
            $quesansGroup = $this->quesansService->findGroupById($topicDto->groupId);
            $quesansSection = $this->quesansService->findSectionById($quesansGroup->sectionId);

            if ( $quesansSection->isHidden )
            {
                $eParams = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'edit_topic');
                $event = new OW_Event('quesans.check_permissions', $eParams);
                OW::getEventManager()->trigger($event);

                if ( $event->getData() )
                {
                    $isModerator = true;
                }
            }

            if ( $isModerator )
            {
                $topicDto->sticky = ($topicDto->sticky) ? 0 : 1;
                $this->quesansService->saveOrUpdateTopic($topicDto);
            }
        }

        $topicUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicId));

        $this->redirect($topicUrl . "?page=$page");
    }

    /**
     * This action locks or unlocks the topic
     * and after execution redirects to default action
     *
     * @param array $params
     * @throws Redirect404Exception
     */
    public function lockTopic( array $params )
    {
        if ( !isset($params['topicId']) || !($topicId = (int) $params['topicId']) || !isset($params['page']) || !($page = (int) $params['page']) )
        {
            throw new Redirect404Exception();
        }

        $isModerator = OW::getUser()->isAuthorized('quesans');

        $topicDto = $this->quesansService->findTopicById($topicId);

        if ( $topicDto )
        {
            $quesansGroup = $this->quesansService->findGroupById($topicDto->groupId);
            $quesansSection = $this->quesansService->findSectionById($quesansGroup->sectionId);

            if ( $quesansSection->isHidden )
            {
                $eParams = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'edit_topic');
                $event = new OW_Event('quesans.check_permissions', $eParams);
                OW::getEventManager()->trigger($event);

                if ( $event->getData() )
                {
                    $isModerator = true;
                }
            }

            if ( $isModerator )
            {
                $topicDto->locked = ($topicDto->locked) ? 0 : 1;
                $this->quesansService->saveOrUpdateTopic($topicDto);
            }
        }

        $topicUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicId));

        $this->redirect($topicUrl . "?page=$page");
    }

    /**
     * This action deletes the topic
     * and after execution redirects to default action
     *
     * @param array $params
     * @throws Redirect404Exception
     */
    public function deleteTopic( array $params )
    {
        if ( !isset($params['topicId']) || !($topicId = (int) $params['topicId']) )
        {
            throw new Redirect404Exception();
        }

        $isModerator = OW::getUser()->isAuthorized('quesans');

        $topicDto = $this->quesansService->findTopicById($topicId);
        $userId = OW::getUser()->getId();

        $redirectUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicId));

        if ( $topicDto )
        {
            $quesansGroup = $this->quesansService->findGroupById($topicDto->groupId);
            $quesansSection = $this->quesansService->findSectionById($quesansGroup->sectionId);

            if ( $quesansSection->isHidden )
            {
                $eParams = array('entity' => $quesansSection->entity, 'entityId' => $quesansGroup->entityId, 'action' => 'edit_topic');
                $event = new OW_Event('quesans.check_permissions', $eParams);
                OW::getEventManager()->trigger($event);

                if ( $event->getData() )
                {
                    $isModerator = true;
                }
            }

            if ( $isModerator || $userId == $topicDto->userId )
            {
                $groupId = $topicDto->groupId;
                $this->quesansService->deleteTopic($topicId);

                $redirectUrl = OW::getRouter()->urlForRoute('group-default', array('groupId' => $groupId));
            }
        }

        $this->redirect($redirectUrl);
    }

    /**
     * This action gets the post called by ajax request
     *
     * @param array $params
     * @throws Redirect404Exception
     */
    public function getPost( array $params )
    {
        if ( isset($params['postId']) && $postId = (int) $params['postId'] )
        {
            if ( OW::getRequest()->isAjax() )
            {
                $postDto = $this->quesansService->findPostById($postId);

                $post = array(
                    'from' => BOL_UserService::getInstance()->getDisplayName($postDto->userId),
                    'text' => $postDto->text
                );

                echo json_encode($post);
            }
            else
            {
                throw new Redirect404Exception();
            }
        }

        exit();
    }

    public function subscribeTopic( $params )
    {
        if ( !empty($params['id']) )
        {
            $subscribeService = QUESANS_BOL_SubscriptionService::getInstance();
            $userId = OW::getUser()->getId();
            $topicId = (int) $params['id'];

            if ( OW::getUser()->isAuthorized('quesans', 'subscribe') && !$subscribeService->isUserSubscribed($userId, $topicId) )
            {
                $subscription = new QUESANS_BOL_Subscription;
                $subscription->userId = $userId;
                $subscription->topicId = $topicId;

                $subscribeService->addSubscription($subscription);

                echo json_encode(array('msg' => OW::getLanguage()->text('quesans', 'subscription-added')));
            }
        }

        exit();
    }

    public function unsubscribeTopic( $params )
    {
        if ( !empty($params['id']) )
        {
            $subscribeService = QUESANS_BOL_SubscriptionService::getInstance();
            $userId = OW::getUser()->getId();
            $topicId = (int) $params['id'];

            if ( $subscribeService->isUserSubscribed($userId, $topicId) )
            {
                $subscribeService->deleteSubscription($userId, $topicId);

                echo json_encode(array('msg' => OW::getLanguage()->text('quesans', 'subscription-canceled')));
            }
        }

        exit();
    }

    /**
     * This action moves the topic called by ajax request
     */
    public function moveTopic()
    {
        $userId = OW::getUser()->getId();
        $isModerator = OW::getUser()->isAuthorized('quesans');

        if ( OW::getRequest()->isAjax() && $_POST['topic-id'] )
        {
            $topicId = (int) $_POST['topic-id'];
            $groupId = (int) $_POST['group-id'];

            $groupDto = $this->quesansService->findGroupById($groupId);
            $topicDto = $this->quesansService->findTopicById($topicId);

            if ( $groupDto === null || $topicDto === null || !$isModerator )
            {
                exit();
            }

            //create replace topic
            $replaceTopicDto = new QUESANS_BOL_Topic();

            $replaceTopicDto->groupId = $topicDto->groupId;
            $replaceTopicDto->userId = $userId;
            $replaceTopicDto->title = $topicDto->title;
            $replaceTopicDto->locked = 1;
            $replaceTopicDto->temp = 1;

            $this->quesansService->saveOrUpdateTopic($replaceTopicDto);

            $oldGroupDto = $this->quesansService->findGroupById($topicDto->groupId);

            $topicUrl = OW::getRouter()->urlForRoute('topic-default', array('topicId' => $topicDto->id));
            $oldGroupUrl = OW::getRouter()->urlForRoute('group-default', array('groupId' => $topicDto->groupId));

            //create replace topic's post
            $replacePostDto = new QUESANS_BOL_Post();

            $replacePostDto->topicId = $replaceTopicDto->id;
            $replacePostDto->userId = $userId;
            $replacePostDto->createStamp = time();
            $replacePostDto->text = OW::getLanguage()->text('quesans', 'moved_to', array('topicUrl' => $topicUrl));

            $this->quesansService->saveOrUpdatePost($replacePostDto);

            $replaceTopicDto->lastPostId = $replacePostDto->id;
            $this->quesansService->saveOrUpdateTopic($replaceTopicDto);

            //create notification post
            $postDto = new QUESANS_BOL_Post();

            $postDto->topicId = $topicDto->id;
            $postDto->userId = $userId;
            $postDto->createStamp = time();
            $postDto->text = OW::getLanguage()->text('quesans', 'moved_from', array('groupUrl' => $oldGroupUrl, 'groupName' => $oldGroupDto->name));

            $this->quesansService->saveOrUpdatePost($postDto);

            $topicDto->groupId = $groupDto->id;
            $topicDto->lastPostId = $postDto->id;

            $this->quesansService->saveOrUpdateTopic($topicDto);

            echo json_encode($this->quesansService->getPostUrl($replaceTopicDto->id, $replacePostDto->id, false));
        }
        else
        {
            throw new Redirect404Exception();
        }

        exit();
    }

    /**
     * Generates add post form.
     *
     * @param int $topicId
     * @param string $uid
     * @return Form
     */
    private function generateAddPostForm( $topicId, $uid )
    {
        $form = new Form('add-post-form');
        $form->setEnctype('multipart/form-data');

        $lang = OW::getLanguage();

        $addPostUrl = OW::getRouter()->urlForRoute('add-post', array('topicId' => $topicId, 'uid' => $uid));
        $form->setAction($addPostUrl);

        $topicIdField = new HiddenField('topic');
        $topicIdField->setValue($topicId);
        $form->addElement($topicIdField);

        $attachmentUid = new HiddenField('attachmentUid');
        $attachmentUid->setValue($uid);
        $attachmentUid->setRequired(true);
        $form->addElement($attachmentUid);

        $btnSet = array(BOL_TextFormatService::WS_BTN_IMAGE, BOL_TextFormatService::WS_BTN_VIDEO, BOL_TextFormatService::WS_BTN_HTML);
        $postText = new WysiwygTextarea('text', $btnSet);
        $postText->setRequired(true);
        $sValidator = new StringValidator(1, 50000);
        $sValidator->setErrorMessage($lang->text('quesans', 'chars_limit_exceeded', array('limit' => 50000)));
        $postText->addValidator($sValidator);
        $form->addElement($postText);

        $submit = new Submit('submit');
        $submit->setValue($lang->text('quesans', 'add_post_btn'));
        $form->addElement($submit);

        return $form;
    }

    /**
     * Generates move topic form.
     *
     * @param string $actionUrl
     * @param $groupSelect
     * @param $topicDto
     * @return Form
     */
    private function generateMoveTopicForm( $actionUrl, $groupSelect, $topicDto )
    {
        $form = new Form('move-topic-form');

        $form->setAction($actionUrl);

        $topicIdField = new HiddenField('topic-id');
        $topicIdField->setValue($topicDto->id);
        $form->addElement($topicIdField);

        $group = new QuesansSelectBox('group-id');
        $group->setOptions($groupSelect);
        $group->setValue($topicDto->groupId);
        $group->addAttribute("style", "width: 300px;");
        $group->setRequired(true);
        $form->addElement($group);

        $submit = new Submit('save');
        $submit->setValue(OW::getLanguage()->text('quesans', 'move_topic_btn'));
        $form->addElement($submit);

        $form->setAjax(true);

        return $form;
    }

    public function approve( $params )
    {
        if ( !OW::getUser()->isAuthorized('quesans') )
        {
            exit();
        }

        $entityId = $params['id'];

        $backUrl = OW::getRouter()->urlForRoute('topic-default', array(
            'topicId' => $entityId
        ));

        $event = new OW_Event("moderation.approve", array(
            "entityType" => QUESANS_CLASS_ContentProvider::ENTITY_TYPE,
            "entityId" => $entityId
        ));

        OW::getEventManager()->trigger($event);

        $data = $event->getData();

        if ( empty($data) )
        {
            $this->redirect($backUrl);
        }

        if ( $data["message"] )
        {
            OW::getFeedback()->info($data["message"]);
        }
        else
        {
            OW::getFeedback()->error($data["error"]);
        }

        $this->redirect($backUrl);
    }
}
