<?php
class QUESANS_MCLASS_EventHandler
{
    /**
     * @var QUESANS_MCLASS_EventHandler
     */
    private static $classInstance;

    /**
     * @return QUESANS_MCLASS_EventHandler
     */
    public static function getInstance()
    {
        if ( self::$classInstance === null )
        {
            self::$classInstance = new self();
        }

        return self::$classInstance;
    }

    private function __construct() { }

    public function feedOnItemRender( OW_Event $event )
    {
        $params = $event->getParams();
        $data = $event->getData();
        $language = OW::getLanguage();

        if ( $params['action']['entityType'] != 'quesans-topic' )
        {
            return;
        }

        $service = QUESANS_BOL_QuesansService::getInstance();
        $postCount = $service->findTopicPostCount($params['action']['entityId']) - 1;

        if ( !$postCount )
        {
            return;
        }

        $postIds = array();
        foreach ( $params['activity'] as $activity )
        {
            if ( $activity['activityType'] == 'quesans-post' )
            {
                $postIds[] = $activity['data']['postId'];
            }
        }

        if ( empty($postIds) )
        {
            return;
        }

        $postDto = null;
        foreach ( $postIds as $pid )
        {
            $postDto = $service->findPostById($pid);
            if ( $postDto !== null )
            {
                break;
            }
        }

        if ( $postDto === null )
        {
            return;
        }

        $postUrlEmbed = '...';
        $content = UTIL_String::truncate(strip_tags(str_replace("&nbsp;", "", $postDto->text)), 100, $postUrlEmbed);
        $usersData = BOL_AvatarService::getInstance()->getDataForUserAvatars(array($postDto->userId), true, true, true, false);

        $avatarData = $usersData[$postDto->userId];
        //$postUrl = $service->getPostUrl($postDto->topicId, $postDto->id);

        if ( is_array($data['content']) && !empty($data['content']['vars']) )
        {
            $data['content']['vars']['activity'] = array(
                'title' => $language->text('quesans', 'latest_reply_from', array('url' => $avatarData['url'], 'user' => $avatarData['title'])),
                'avatarData' => $avatarData,
                'description' => $content
            );
        }

        $event->setData($data);
    }

    public function onFeedItemRenderDisableActions( OW_Event $event )
    {
        $params = $event->getParams();

        if ( !in_array($params["action"]["entityType"], array("quesans-topic", "quesans-post")) )
        {
            return;
        }

        $data = $event->getData();

        $data["disabled"] = true;

        $event->setData($data);
    }

    public function init()
    {
        QUESANS_CLASS_EventHandler::getInstance()->genericInit();

        $em = OW::getEventManager();

        $em->bind('feed.on_item_render', array($this, 'feedOnItemRender'));
        $em->bind('feed.on_item_render', array($this, 'onFeedItemRenderDisableActions'));
    }
}